@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) University of Cambridge 1995 - 2004 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This file, which provides support for Microsoft's Secure Password
@@ -12,7 +12,6 @@ server support. I (PH) have only modified it in very trivial ways.
 References:
   http://www.innovation.ch/java/ntlm.html
   http://www.kuro5hin.org/story/2002/4/28/1436/66154
-  http://download.microsoft.com/download/9/5/e/95ef66af-9026-4bb0-a41d-a4f81802d92c/%5bMS-SMTP%5d.pdf
 
  * It seems that some systems have existing but different definitions of some
  * of the following types. I received a complaint about "int16" causing
@@ -24,10 +23,8 @@ References:
  * typedef unsigned uint32;
  * typedef unsigned char  uint8;
 
-07-August-2003:  PH: Patched up the code to avoid assert bombouts for stupid
-                     input data. Find appropriate comment by grepping for "PH".
-16-October-2006: PH: Added a call to auth_check_serv_cond() at the end
-05-June-2010:    PP: handle SASL initial response
+07-August-2003: PH: Patched up the code to avoid assert bombouts for stupid
+                    input data. Find appropriate comment by grepping for "PH".
 */
 
 
@@ -61,7 +58,7 @@ address can appear in the tables drtables.c. */
 int auth_spa_options_count =
   sizeof(auth_spa_options)/sizeof(optionlist);
 
-/* Default private options block for the condition authentication method. */
+/* Default private options block for the contidion authentication method. */
 
 auth_spa_options_block auth_spa_option_defaults = {
   NULL,              /* spa_password */
@@ -71,19 +68,6 @@ auth_spa_options_block auth_spa_option_defaults = {
 };
 
 
-#ifdef MACRO_PREDEF
-
-/* Dummy values */
-void auth_spa_init(auth_instance *ablock) {}
-int auth_spa_server(auth_instance *ablock, uschar *data) {return 0;}
-int auth_spa_client(auth_instance *ablock, void * sx, int timeout,
-    uschar *buffer, int buffsize) {return 0;}
-
-#else   /*!MACRO_PREDEF*/
-
-
-
-
 /*************************************************
 *          Initialization entry point            *
 *************************************************/
@@ -123,7 +107,7 @@ ablock->server = ob->spa_serverpassword != NULL;
 
 /* For interface, see auths/README */
 
-#define CVAL(buf,pos) ((US (buf))[pos])
+#define CVAL(buf,pos) (((unsigned char *)(buf))[pos])
 #define PVAL(buf,pos) ((unsigned)CVAL(buf,pos))
 #define SVAL(buf,pos) (PVAL(buf,pos)|PVAL(buf,(pos)+1)<<8)
 #define IVAL(buf,pos) (SVAL(buf,pos)|SVAL(buf,(pos)+2)<<16)
@@ -139,20 +123,17 @@ SPAAuthChallenge challenge;
 SPAAuthResponse  response;
 SPAAuthResponse  *responseptr = &response;
 uschar msgbuf[2048];
-uschar *clearpass, *s;
-unsigned off;
+uschar *clearpass;
 
-/* send a 334, MS Exchange style, and grab the client's request,
-unless we already have it via an initial response. */
+/* send a 334, MS Exchange style, and grab the client's request */
 
-if ((*data == '\0') &&
-    (auth_get_no64_data(&data, US"NTLM supported") != OK))
+if (auth_get_no64_data(&data, US"NTLM supported") != OK)
   {
   /* something borked */
   return FAIL;
   }
 
-if (spa_base64_to_bits(CS (&request), sizeof(request), CCS (data)) < 0)
+if (spa_base64_to_bits((char *)(&request), sizeof(request), (const char *)(data)) < 0)
   {
   DEBUG(D_auth) debug_printf("auth_spa_server(): bad base64 data in "
   "request: %s\n", data);
@@ -172,13 +153,15 @@ if (auth_get_no64_data(&data, msgbuf) != OK)
   }
 
 /* dump client response */
-if (spa_base64_to_bits(CS (&response), sizeof(response), CCS (data)) < 0)
+if (spa_base64_to_bits((char *)(&response), sizeof(response), (const char *)(data)) < 0)
   {
   DEBUG(D_auth) debug_printf("auth_spa_server(): bad base64 data in "
   "response: %s\n", data);
   return FAIL;
   }
 
+/* get username and put it in $1 */
+
 /***************************************************************
 PH 07-Aug-2003: The original code here was this:
 
@@ -195,19 +178,9 @@ that causes failure if the size of msgbuf is exceeded. ****/
 
   {
   int i;
-  char * p;
+  char *p = ((char*)responseptr) + IVAL(&responseptr->uUser.offset,0);
   int len = SVAL(&responseptr->uUser.len,0)/2;
 
-  if (  (off = IVAL(&responseptr->uUser.offset,0)) >= sizeof(SPAAuthResponse)
-     || len >= sizeof(responseptr->buffer)/2
-     || (p = (CS responseptr) + off) + len*2 >= CS (responseptr+1)
-     )
-    {
-    DEBUG(D_auth)
-      debug_printf("auth_spa_server(): bad uUser spec in response\n");
-    return FAIL;
-    }
-
   if (len + 1 >= sizeof(msgbuf)) return FAIL;
   for (i = 0; i < len; ++i)
     {
@@ -219,23 +192,16 @@ that causes failure if the size of msgbuf is exceeded. ****/
 
 /***************************************************************/
 
-/* Put the username in $auth1 and $1. The former is now the preferred variable;
-the latter is the original variable. These have to be out of stack memory, and
-need to be available once known even if not authenticated, for error messages
-(server_set_id, which only makes it to authenticated_id if we return OK) */
-
-auth_vars[0] = expand_nstring[1] = string_copy(msgbuf);
+expand_nstring[1] = msgbuf;
 expand_nlength[1] = Ustrlen(msgbuf);
 expand_nmax = 1;
 
-debug_print_string(ablock->server_debug_string);    /* customized debug */
-
 /* look up password */
 
 clearpass = expand_string(ob->spa_serverpassword);
 if (clearpass == NULL)
   {
-  if (f.expand_string_forcedfail)
+  if (expand_string_forcedfail)
     {
     DEBUG(D_auth) debug_printf("auth_spa_server(): forced failure while "
       "expanding spa_serverpassword\n");
@@ -256,19 +222,11 @@ spa_smb_nt_encrypt (clearpass, challenge.challengeData, ntRespData);
 
 /* compare NT hash (LM may not be available) */
 
-off = IVAL(&responseptr->ntResponse.offset,0);
-if (off >= sizeof(SPAAuthResponse) - 24)
-  {
-  DEBUG(D_auth)
-    debug_printf("auth_spa_server(): bad ntRespData spec in response\n");
-  return FAIL;
-  }
-s = (US responseptr) + off;
-
-if (memcmp(ntRespData, s, 24) == 0)
-  return auth_check_serv_cond(ablock);	/* success. we have a winner. */
-
-  /* Expand server_condition as an authorization check (PH) */
+if (memcmp(ntRespData,
+      ((unsigned char*)responseptr)+IVAL(&responseptr->ntResponse.offset,0),
+      24) == 0)
+  /* success. we have a winner. */
+  return OK;
 
 return FAIL;
 }
@@ -283,106 +241,113 @@ return FAIL;
 int
 auth_spa_client(
   auth_instance *ablock,                 /* authenticator block */
-  void * sx,				 /* connection */
+  smtp_inblock *inblock,                 /* connection inblock */
+  smtp_outblock *outblock,               /* connection outblock */
   int timeout,                           /* command timeout */
   uschar *buffer,                        /* buffer for reading response */
   int buffsize)                          /* size of buffer */
 {
-auth_spa_options_block *ob =
-       (auth_spa_options_block *)(ablock->options_block);
-SPAAuthRequest   request;
-SPAAuthChallenge challenge;
-SPAAuthResponse  response;
-char msgbuf[2048];
-char *domain = NULL;
-char *username, *password;
+       auth_spa_options_block *ob =
+               (auth_spa_options_block *)(ablock->options_block);
+       SPAAuthRequest   request;
+       SPAAuthChallenge challenge;
+       SPAAuthResponse  response;
+       char msgbuf[2048];
+       char *domain = NULL;
+       char *username, *password;
 
-/* Code added by PH to expand the options */
+       /* Code added by PH to expand the options */
 
-*buffer = 0;    /* Default no message when cancelled */
+       username = CS expand_string(ob->spa_username);
+       if (username == NULL)
+         {
+         if (expand_string_forcedfail) return CANCELLED;
+         string_format(buffer, buffsize, "expansion of \"%s\" failed in %s "
+           "authenticator: %s", ob->spa_username, ablock->name,
+           expand_string_message);
+         return ERROR;
+         }
 
-if (!(username = CS expand_string(ob->spa_username)))
-  {
-  if (f.expand_string_forcedfail) return CANCELLED;
-  string_format(buffer, buffsize, "expansion of \"%s\" failed in %s "
-   "authenticator: %s", ob->spa_username, ablock->name,
-   expand_string_message);
-  return ERROR;
-  }
+       password = CS expand_string(ob->spa_password);
+       if (password == NULL)
+         {
+         if (expand_string_forcedfail) return CANCELLED;
+         string_format(buffer, buffsize, "expansion of \"%s\" failed in %s "
+           "authenticator: %s", ob->spa_password, ablock->name,
+           expand_string_message);
+         return ERROR;
+         }
 
-if (!(password = CS expand_string(ob->spa_password)))
-  {
-  if (f.expand_string_forcedfail) return CANCELLED;
-  string_format(buffer, buffsize, "expansion of \"%s\" failed in %s "
-   "authenticator: %s", ob->spa_password, ablock->name,
-   expand_string_message);
-  return ERROR;
-  }
+       if (ob->spa_domain != NULL)
+         {
+         domain = CS expand_string(ob->spa_domain);
+         if (domain == NULL)
+           {
+           if (expand_string_forcedfail) return CANCELLED;
+           string_format(buffer, buffsize, "expansion of \"%s\" failed in %s "
+             "authenticator: %s", ob->spa_domain, ablock->name,
+             expand_string_message);
+           return ERROR;
+           }
+         }
 
-if (ob->spa_domain)
-  if (!(domain = CS expand_string(ob->spa_domain)))
-    {
-    if (f.expand_string_forcedfail) return CANCELLED;
-    string_format(buffer, buffsize, "expansion of \"%s\" failed in %s "
-		  "authenticator: %s", ob->spa_domain, ablock->name,
-		  expand_string_message);
-    return ERROR;
-    }
+       /* Original code */
 
-/* Original code */
+    if (smtp_write_command(outblock, FALSE, "AUTH %s\r\n",
+         ablock->public_name) < 0)
+               return FAIL_SEND;
 
-if (smtp_write_command(sx, SCMD_FLUSH, "AUTH %s\r\n", ablock->public_name) < 0)
-  return FAIL_SEND;
+       /* wait for the 3XX OK message */
+       if (!smtp_read_response(inblock, (uschar *)buffer, buffsize, '3', timeout))
+               return FAIL;
 
-/* wait for the 3XX OK message */
-if (!smtp_read_response(sx, US buffer, buffsize, '3', timeout))
-  return FAIL;
+       DSPA("\n\n%s authenticator: using domain %s\n\n",
+               ablock->name, domain);
 
-DSPA("\n\n%s authenticator: using domain %s\n\n", ablock->name, domain);
+       spa_build_auth_request (&request, CS username, domain);
+       spa_bits_to_base64 (US msgbuf, (unsigned char*)&request,
+               spa_request_length(&request));
 
-spa_build_auth_request (&request, CS username, domain);
-spa_bits_to_base64 (US msgbuf, (unsigned char*)&request,
-       spa_request_length(&request));
+       DSPA("\n\n%s authenticator: sending request (%s)\n\n", ablock->name,
+               msgbuf);
 
-DSPA("\n\n%s authenticator: sending request (%s)\n\n", ablock->name, msgbuf);
+       /* send the encrypted password */
+       if (smtp_write_command(outblock, FALSE, "%s\r\n", msgbuf) < 0)
+               return FAIL_SEND;
 
-/* send the encrypted password */
-if (smtp_write_command(sx, SCMD_FLUSH, "%s\r\n", msgbuf) < 0)
-  return FAIL_SEND;
+       /* wait for the auth challenge */
+       if (!smtp_read_response(inblock, (uschar *)buffer, buffsize, '3', timeout))
+               return FAIL;
 
-/* wait for the auth challenge */
-if (!smtp_read_response(sx, US buffer, buffsize, '3', timeout))
-  return FAIL;
+       /* convert the challenge into the challenge struct */
+       DSPA("\n\n%s authenticator: challenge (%s)\n\n",
+               ablock->name, buffer + 4);
+       spa_base64_to_bits ((char *)(&challenge), sizeof(challenge), (const char *)(buffer + 4));
 
-/* convert the challenge into the challenge struct */
-DSPA("\n\n%s authenticator: challenge (%s)\n\n", ablock->name, buffer + 4);
-spa_base64_to_bits (CS (&challenge), sizeof(challenge), CCS (buffer + 4));
+       spa_build_auth_response (&challenge, &response,
+               CS username, CS password);
+       spa_bits_to_base64 (US msgbuf, (unsigned char*)&response,
+               spa_request_length(&response));
+       DSPA("\n\n%s authenticator: challenge response (%s)\n\n", ablock->name,
+               msgbuf);
 
-spa_build_auth_response (&challenge, &response, CS username, CS password);
-spa_bits_to_base64 (US msgbuf, (unsigned char*)&response,
-       spa_request_length(&response));
-DSPA("\n\n%s authenticator: challenge response (%s)\n\n", ablock->name, msgbuf);
+       /* send the challenge response */
+       if (smtp_write_command(outblock, FALSE, "%s\r\n", msgbuf) < 0)
+               return FAIL_SEND;
 
-/* send the challenge response */
-if (smtp_write_command(sx, SCMD_FLUSH, "%s\r\n", msgbuf) < 0)
-       return FAIL_SEND;
+       /* If we receive a success response from the server, authentication
+       has succeeded. There may be more data to send, but is there any point
+       in provoking an error here? */
+       if (smtp_read_response(inblock, US buffer, buffsize, '2', timeout))
+               return OK;
 
-/* If we receive a success response from the server, authentication
-has succeeded. There may be more data to send, but is there any point
-in provoking an error here? */
+       /* Not a success response. If errno != 0 there is some kind of transmission
+       error. Otherwise, check the response code in the buffer. If it starts with
+       '3', more data is expected. */
+       if (errno != 0 || buffer[0] != '3')
+               return FAIL;
 
-if (smtp_read_response(sx, US buffer, buffsize, '2', timeout))
-  return OK;
-
-/* Not a success response. If errno != 0 there is some kind of transmission
-error. Otherwise, check the response code in the buffer. If it starts with
-'3', more data is expected. */
-
-if (errno != 0 || buffer[0] != '3')
-  return FAIL;
-
-return FAIL;
+       return FAIL;
 }
 
-#endif   /*!MACRO_PREDEF*/
 /* End of spa.c */
