@@ -1,10 +1,8 @@
-/* $Cambridge: exim/exim-src/src/child.c,v 1.12 2009/11/16 19:50:36 nm4 Exp $ */
-
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2004 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -32,13 +30,13 @@ static void
 force_fd(int oldfd, int newfd)
 {
 if (oldfd == newfd) return;
-(void)close(newfd);
-(void)dup2(oldfd, newfd);
-(void)close(oldfd);
+close(newfd);
+dup2(oldfd, newfd);
+close(oldfd);
 }
 
 
-#ifndef STAND_ALONE
+
 /*************************************************
 *   Build argv list and optionally re-exec Exim  *
 *************************************************/
@@ -174,11 +172,6 @@ the new process, and returns that to the caller via fdptr. The function returns
 the pid of the new process, or -1 if things go wrong. If debug_fd is
 non-negative, it is passed as stderr.
 
-This interface is now a just wrapper for the more complicated function
-child_open_exim2(), which has additional arguments. The wrapper must continue
-to exist, even if all calls from within Exim are changed, because it is
-documented for use from local_scan().
-
 Argument: fdptr   pointer to int for the stdin fd
 Returns:          pid of the created process or -1 if anything has gone wrong
 */
@@ -186,24 +179,6 @@ Returns:          pid of the created process or -1 if anything has gone wrong
 pid_t
 child_open_exim(int *fdptr)
 {
-return child_open_exim2(fdptr, US"<>", bounce_sender_authentication);
-}
-
-
-/* This is a more complicated function for creating a child Exim process, with
-more arguments.
-
-Arguments:
-  fdptr                   pointer to int for the stdin fd
-  sender                  for a sender address (data for -f)
-  sender_authentication   authenticated sender address or NULL
-
-Returns:          pid of the created process or -1 if anything has gone wrong
-*/
-
-pid_t
-child_open_exim2(int *fdptr, uschar *sender, uschar *sender_authentication)
-{
 int pfd[2];
 int save_errno;
 pid_t pid;
@@ -219,46 +194,28 @@ pid = fork();
 
 /* Child process: make the reading end of the pipe into the standard input and
 close the writing end. If debugging, pass debug_fd as stderr. Then re-exec
-Exim with appropriat options. In the test harness, use -odi unless queue_only
-is set, so that the bounce is fully delivered before returning. Failure is
-signalled with EX_EXECFAILED (specified by CEE_EXEC_EXIT), but this shouldn't
-occur. */
+Exim. Failure is signalled with EX_EXECFAILED, but this shouldn't occur! */
 
 if (pid == 0)
   {
   force_fd(pfd[pipe_read], 0);
-  (void)close(pfd[pipe_write]);
+  close(pfd[pipe_write]);
   if (debug_fd > 0) force_fd(debug_fd, 2);
-  if (running_in_test_harness && !queue_only)
-    {
-    if (sender_authentication != NULL)
-      child_exec_exim(CEE_EXEC_EXIT, FALSE, NULL, FALSE, 9,
-        US "-odi", US"-t", US"-oem", US"-oi", US"-f", sender, US"-oMas",
-        sender_authentication, message_id_option);
-    else
-      child_exec_exim(CEE_EXEC_EXIT, FALSE, NULL, FALSE, 7,
-        US "-odi", US"-t", US"-oem", US"-oi", US"-f", sender,
-        message_id_option);
-    /* Control does not return here. */
-    }
-  else   /* Not test harness */
-    {
-    if (sender_authentication != NULL)
-      child_exec_exim(CEE_EXEC_EXIT, FALSE, NULL, FALSE, 8,
-        US"-t", US"-oem", US"-oi", US"-f", sender, US"-oMas",
-        sender_authentication, message_id_option);
-    else
-      child_exec_exim(CEE_EXEC_EXIT, FALSE, NULL, FALSE, 6,
-        US"-t", US"-oem", US"-oi", US"-f", sender, message_id_option);
-    /* Control does not return here. */
-    }
+  if (bounce_sender_authentication != NULL)
+    child_exec_exim(CEE_EXEC_EXIT, FALSE, NULL, FALSE, 8,
+      US"-t", US"-oem", US"-oi", US"-f", US"<>", US"-oMas",
+      bounce_sender_authentication, message_id_option);
+  else
+    child_exec_exim(CEE_EXEC_EXIT, FALSE, NULL, FALSE, 6,
+      US"-t", US"-oem", US"-oi", US"-f", US"<>", message_id_option);
+  /* Control does not return here. */
   }
 
 /* Parent process. Save fork() errno and close the reading end of the stdin
 pipe. */
 
 save_errno = errno;
-(void)close(pfd[pipe_read]);
+close(pfd[pipe_read]);
 
 /* Fork succeeded */
 
@@ -270,11 +227,11 @@ if (pid > 0)
 
 /* Fork failed */
 
-(void)close(pfd[pipe_write]);
+close(pfd[pipe_write]);
 errno = save_errno;
 return (pid_t)(-1);
 }
-#endif   /* STAND_ALONE */
+
 
 
 
@@ -304,7 +261,7 @@ Arguments:
                 process is placed
   wd          if not NULL, a path to be handed to chdir() in the new process
   make_leader if TRUE, make the new process a process group leader
-
+  
 Returns:      the pid of the created process or -1 if anything has gone wrong
 */
 
@@ -321,8 +278,8 @@ pid_t pid;
 if (pipe(inpfd) != 0) return (pid_t)(-1);
 if (pipe(outpfd) != 0)
   {
-  (void)close(inpfd[pipe_read]);
-  (void)close(inpfd[pipe_write]);
+  close(inpfd[pipe_read]);
+  close(inpfd[pipe_write]);
   return (pid_t)(-1);
   }
 
@@ -333,57 +290,34 @@ otherwise. Save the old state for resetting on the wait. */
 oldsignal = signal(SIGCHLD, SIG_DFL);
 pid = fork();
 
-/* Handle the child process. First, set the required environment. We must do
-this before messing with the pipes, in order to be able to write debugging
-output when things go wrong. */
+/* The child process becomes a process group leader if requested, and then
+organizes the pipes. Any unexpected failure is signalled with EX_EXECFAILED;
+these are all "should never occur" failures, except perhaps for exec failing
+because the command doesn't exist. */
 
 if (pid == 0)
   {
-  signal(SIGUSR1, SIG_IGN);
+  if (make_leader && setpgid(0,0) < 0) goto CHILD_FAILED;
 
-  if (newgid != NULL && setgid(*newgid) < 0)
-    {
-    DEBUG(D_any) debug_printf("failed to set gid=%ld in subprocess: %s\n",
-      (long int)(*newgid), strerror(errno));
-    goto CHILD_FAILED;
-    }
-
-  if (newuid != NULL && setuid(*newuid) < 0)
-    {
-    DEBUG(D_any) debug_printf("failed to set uid=%ld in subprocess: %s\n",
-      (long int)(*newuid), strerror(errno));
-    goto CHILD_FAILED;
-    }
-
-  (void)umask(newumask);
-
-  if (wd != NULL && Uchdir(wd) < 0)
-    {
-    DEBUG(D_any) debug_printf("failed to chdir to %s: %s\n", wd,
-      strerror(errno));
-    goto CHILD_FAILED;
-    }
-
-  /* Becomes a process group leader if requested, and then organize the pipes.
-  Any unexpected failure is signalled with EX_EXECFAILED; these are all "should
-  never occur" failures, except for exec failing because the command doesn't
-  exist. */
-
-  if (make_leader && setpgid(0,0) < 0)
-    {
-    DEBUG(D_any) debug_printf("failed to set group leader in subprocess: %s\n",
-      strerror(errno));
-    goto CHILD_FAILED;
-    }
-
-  (void)close(inpfd[pipe_write]);
+  close(inpfd[pipe_write]);
   force_fd(inpfd[pipe_read], 0);
 
-  (void)close(outpfd[pipe_read]);
+  close(outpfd[pipe_read]);
   force_fd(outpfd[pipe_write], 1);
 
-  (void)close(2);
-  (void)dup2(1, 2);
+  close(2);
+  dup2(1, 2);
+
+  /* Set the required environment. */
+
+  signal(SIGUSR1, SIG_IGN);
+  if (newgid != NULL && setgid(*newgid) < 0) goto CHILD_FAILED;
+  if (newuid != NULL && setuid(*newuid) < 0) goto CHILD_FAILED;
+  (void)umask(newumask);
+
+  /* Set the working directory if required */
+
+  if (wd != NULL && Uchdir(wd) < 0) goto CHILD_FAILED;
 
   /* Now do the exec */
 
@@ -392,7 +326,7 @@ if (pid == 0)
 
   /* Failed to execv. Signal this failure using EX_EXECFAILED. We are
   losing the actual errno we got back, because there is no way to return
-  this information. */
+  this. */
 
   CHILD_FAILED:
   _exit(EX_EXECFAILED);      /* Note: must be _exit(), NOT exit() */
@@ -402,8 +336,8 @@ if (pid == 0)
 stdin pipe, and the writing end of the stdout pipe. */
 
 save_errno = errno;
-(void)close(inpfd[pipe_read]);
-(void)close(outpfd[pipe_write]);
+close(inpfd[pipe_read]);
+close(outpfd[pipe_write]);
 
 /* Fork succeeded; return the input/output pipes and the pid */
 
@@ -416,8 +350,8 @@ if (pid > 0)
 
 /* Fork failed; reset fork errno before returning */
 
-(void)close(inpfd[pipe_write]);
-(void)close(outpfd[pipe_read]);
+close(inpfd[pipe_write]);
+close(outpfd[pipe_read]);
 errno = save_errno;
 return (pid_t)(-1);
 }
