@@ -82,8 +82,8 @@ static void
 main_sigchld_handler(int sig)
 {
 sig = sig;    /* Keep picky compilers happy */
+os_non_restarting_signal(SIGCHLD, SIG_DFL);
 sigchld_seen = TRUE;
-signal(SIGCHLD, SIG_DFL);
 }
 
 
@@ -145,6 +145,7 @@ int dup_accept_socket = -1;
 int max_for_this_host = 0;
 int wfsize = 0;
 int wfptr = 0;
+int use_log_write_selector = log_write_selector;
 uschar *whofrom = NULL;
 
 void *reset_point = store_get(0);
@@ -325,18 +326,26 @@ if ((max_for_this_host > 0) &&
 
 /* OK, the connection count checks have been passed. Before we can fork the
 accepting process, we must first log the connection if requested. This logging
-used to happen in the subprocess, but doing that that means that the value of
+used to happen in the subprocess, but doing that means that the value of
 smtp_accept_count can be out of step by the time it is logged. So we have to do
 the logging here and accept the performance cost. Note that smtp_accept_count
 hasn't yet been incremented to take account of this connection.
 
 In order to minimize the cost (because this is going to happen for every
-connection), do a selector test here. This saves ploughing through the
-generalized logging code each time when the selector is false. */
+connection), do a preliminary selector test here. This saves ploughing through
+the generalized logging code each time when the selector is false. If the
+selector is set, check whether the host is on the list for logging. If not,
+arrange to unset the selector in the subprocess. */
 
 if ((log_write_selector & L_smtp_connection) != 0)
-  log_write(0, LOG_MAIN, "SMTP connection from %s "
-    "(TCP/IP connection count = %d)", whofrom, smtp_accept_count + 1);
+  {
+  uschar *list = hosts_connection_nolog;
+  if (list != NULL && verify_check_host(&list) == OK)
+    use_log_write_selector &= ~L_smtp_connection;
+  else
+    log_write(L_smtp_connection, LOG_MAIN, "SMTP connection from %s "
+      "(TCP/IP connection count = %d)", whofrom, smtp_accept_count + 1);
+  }
 
 /* Now we can fork the accepting process; do a lookup tidy, just in case any
 expansion above did a lookup. */
@@ -356,12 +365,20 @@ if (pid == 0)
   struct sigaction act;
   #endif
 
+  /* May have been modified for the subprocess */
+
+  log_write_selector = use_log_write_selector;
+
   /* Get the local interface address into permanent store */
 
   store_pool = POOL_PERM;
   interface_address = string_copy(interface_address);
   store_pool = old_pool;
 
+  /* Check for a tls-on-connect port */
+
+  if (host_is_tls_on_connect_port(interface_port)) tls_on_connect = TRUE;
+
   /* Expand smtp_active_hostname if required. We do not do this any earlier,
   because it may depend on the local interface address (indeed, that is most
   likely what it depends on.) */
@@ -867,7 +884,9 @@ if (daemon_listen)
   order to perform an "open" on the kernel memory file). */
 
   #ifdef LOAD_AVG_NEEDS_ROOT
-  if (queue_only_load >= 0 || smtp_load_reserve >= 0) (void)os_getloadavg();
+  if (queue_only_load >= 0 || smtp_load_reserve >= 0 ||
+       (deliver_queue_load_max >= 0 && deliver_drop_privilege)) 
+    (void)os_getloadavg();
   #endif
 
   /* If -oX was used, disable the writing of a pid file unless -oP was
@@ -1346,7 +1365,7 @@ if (queue_interval > 0 && queue_run_max > 0)
 /* Set up the handler for termination of child processes. */
 
 sigchld_seen = FALSE;
-signal(SIGCHLD, main_sigchld_handler);
+os_non_restarting_signal(SIGCHLD, main_sigchld_handler);
 
 /* If we are to run the queue periodically, pretend the alarm has just gone
 off. This will cause the first queue-runner to get kicked off straight away. */
@@ -1358,7 +1377,9 @@ must be set up. */
 
 if (daemon_listen)
   {
-  int i;
+  int i, j;
+  int smtp_ports = 0;
+  int smtps_ports = 0;
   ip_address_item *ipa;
   uschar *p = big_buffer;
   uschar *qinfo = (queue_interval > 0)?
@@ -1366,44 +1387,74 @@ if (daemon_listen)
     :
     US"no queue runs";
 
-  #ifdef SUPPORT_TLS
-  uschar *sinfo = tls_on_connect ?
-      US"listening for SMTPS on" : US"listening for SMTP on";
-  #else
-  uschar *sinfo = US"listening for SMTP on";
-  #endif
-
   /* Build a list of listening addresses in big_buffer, but limit it to 10
-  items. The style is for backwards compatibility. */
+  items. The style is for backwards compatibility.
 
-  for (i = 0, ipa = addresses; i < 10 && ipa != NULL; i++, ipa = ipa->next)
-     {
-     if (ipa->address[0] == ':' && ipa->address[1] == 0)
+  It is now possible to have some ports listening for SMTPS (the old,
+  deprecated protocol that starts TLS without using STARTTLS), and others
+  listening for standard SMTP. Keep their listings separate. */
+
+  for (j = 0; j < 2; j++)
+    {
+    for (i = 0, ipa = addresses; i < 10 && ipa != NULL; i++, ipa = ipa->next)
        {
-       if (ipa->next != NULL && ipa->next->address[0] == 0 &&
-           ipa->next->port == ipa->port)
-         {
-         (void)sprintf(CS p, " port %d (IPv6 and IPv4)", ipa->port);
-         ipa = ipa->next;
-         }
-       else if (ipa->v6_include_v4)
-         (void)sprintf(CS p, " port %d (IPv6 with IPv4)", ipa->port);
-       else
-         (void)sprintf(CS p, " port %d (IPv6)", ipa->port);
-       }
-     else if (ipa->address[0] == 0)
-       (void)sprintf(CS p, " port %d (IPv4)", ipa->port);
-     else
-       (void)sprintf(CS p, " [%s]:%d", ipa->address, ipa->port);
-     while (*p != 0) p++;
-     }
+       /* First time round, look for SMTP ports; second time round, look for
+       SMTPS ports. For the first one of each, insert leading text. */
 
-  if (ipa != NULL) memcpy(p, " ...", 5);
+       if (host_is_tls_on_connect_port(ipa->port) == (j > 0))
+         {
+         if (j == 0)
+           {
+           if (smtp_ports++ == 0)
+             {
+             memcpy(p, "SMTP on", 8);
+             p += 7;
+             }
+           }
+         else
+           {
+           if (smtps_ports++ == 0)
+             {
+             (void)sprintf(CS p, "%sSMTPS on",
+               (smtp_ports == 0)? "":" and for ");
+             while (*p != 0) p++;
+             }
+           }
+
+         /* Now the information about the port (and sometimes interface) */
+
+         if (ipa->address[0] == ':' && ipa->address[1] == 0)
+           {
+           if (ipa->next != NULL && ipa->next->address[0] == 0 &&
+               ipa->next->port == ipa->port)
+             {
+             (void)sprintf(CS p, " port %d (IPv6 and IPv4)", ipa->port);
+             ipa = ipa->next;
+             }
+           else if (ipa->v6_include_v4)
+             (void)sprintf(CS p, " port %d (IPv6 with IPv4)", ipa->port);
+           else
+             (void)sprintf(CS p, " port %d (IPv6)", ipa->port);
+           }
+         else if (ipa->address[0] == 0)
+           (void)sprintf(CS p, " port %d (IPv4)", ipa->port);
+         else
+           (void)sprintf(CS p, " [%s]:%d", ipa->address, ipa->port);
+         while (*p != 0) p++;
+         }
+       }
+
+    if (ipa != NULL)
+      {
+      memcpy(p, " ...", 5);
+      p += 4;
+      }
+    }
 
   log_write(0, LOG_MAIN,
-    "exim %s daemon started: pid=%d, %s, %s%s",
-    version_string, getpid(), qinfo, sinfo, big_buffer);
-  set_process_info("daemon: %s, %s%s", qinfo, sinfo, big_buffer);
+    "exim %s daemon started: pid=%d, %s, listening for %s",
+    version_string, getpid(), qinfo, big_buffer);
+  set_process_info("daemon: %s, listening for %s", qinfo, big_buffer);
   }
 
 else
@@ -1416,9 +1467,9 @@ else
   }
 
 
-/* Close the log so it can be renamed and moved. This process doesn't write
-to the log again, unless it is about to die or exec and in the latter case
-it closes the log first. */
+/* Close the log so it can be renamed and moved. In the few cases below where
+this long-running process writes to the log (always exceptional conditions), it
+closes the log afterwards, for the same reason. */
 
 log_close_all();
 
@@ -1504,6 +1555,7 @@ for (;;)
         {
         log_write(0, LOG_MAIN|LOG_PANIC, "daemon: fork of queue-runner "
           "process failed: %s", strerror(errno));
+        log_close_all();
         }
       else
         {
@@ -1553,17 +1605,34 @@ for (;;)
       }
 
     DEBUG(D_any) debug_printf("Listening...\n");
+    
+    /* In rare cases we may have had a SIGCHLD signal in the time between 
+    setting the handler (below) and getting back here. If so, pretend that the 
+    select() was interrupted so that we reap the child. This might still leave
+    a small window when a SIGCHLD could get lost. However, since we use SIGCHLD 
+    only to do the reaping more quickly, it shouldn't result in anything other
+    than a delay until something else causes a wake-up. */
 
-    if ((lcount = select(max_socket + 1, (SELECT_ARG2_TYPE *)&select_listen,
-         NULL, NULL, NULL)) < 0)
+    if (sigchld_seen)
+      {
+      lcount = -1;
+      errno = EINTR;  
+      }
+    else
+      {      
+      lcount = select(max_socket + 1, (SELECT_ARG2_TYPE *)&select_listen,
+        NULL, NULL, NULL);
+      }   
+
+    if (lcount < 0)
       {
       select_failed = TRUE;
       lcount = 1;
       }
-
+      
     /* Loop for all the sockets that are currently ready to go. If select
-    actually failed, we have set the count to 1 and a flag, so as to use the
-    common error code for select/accept below. */
+    actually failed, we have set the count to 1 and select_failed=TRUE, so as
+    to use the common error code for select/accept below. */
 
     while (lcount-- > 0)
       {
@@ -1611,6 +1680,7 @@ for (;;)
               accept_retry_select_failed? "select" : "accept",
               (accept_retry_count == 1)? "" : "s",
               strerror(accept_retry_errno));
+            log_close_all();
             accept_retry_count = 0;
             accept_retry_errno = errno;
             accept_retry_select_failed = select_failed;
@@ -1628,6 +1698,7 @@ for (;;)
             accept_retry_select_failed? "select" : "accept",
             (accept_retry_count == 1)? "" : "s",
             strerror(accept_retry_errno));
+          log_close_all();
           accept_retry_count = 0;
           }
         }
@@ -1712,7 +1783,7 @@ for (;;)
   if (sigchld_seen)
     {
     sigchld_seen = FALSE;
-    signal(SIGCHLD, main_sigchld_handler);
+    os_non_restarting_signal(SIGCHLD, main_sigchld_handler);
     }
 
   /* Handle being woken by SIGHUP. We know at this point that the result
@@ -1720,7 +1791,8 @@ for (;;)
   closing the listening sockets so that they can be reused. Cancel any pending
   alarm in case it is just about to go off, and set SIGHUP to be ignored so
   that another HUP in quick succession doesn't clobber the new daemon before it
-  gets going. All log files get closed by the close-on-exec flag. */
+  gets going. All log files get closed by the close-on-exec flag; however, if
+  the exec fails, we need to close the logs. */
 
   if (sighup_seen)
     {
@@ -1735,6 +1807,7 @@ for (;;)
     execv(CS exim_path, (char *const *)sighup_argv);
     log_write(0, LOG_MAIN|LOG_PANIC_DIE, "pid %d: exec of %s failed: %s",
       getpid(), exim_path, strerror(errno));
+    log_close_all();
     }
 
   }   /* End of main loop */
