@@ -1,10 +1,8 @@
-/* $Cambridge: exim/exim-src/src/exim_dbmbuild.c,v 1.8 2007/01/08 10:50:18 ph10 Exp $ */
-
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2007 */
+/* Copyright (c) University of Cambridge 1995 - 2004 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -58,18 +56,12 @@ return sys_errlist[n];
 
 /* For Berkeley DB >= 2, we can define a function to be called in case of DB
 errors. This should help with debugging strange DB problems, e.g. getting "File
-exists" when you try to open a db file. The API changed at release 4.3. */
+exists" when you try to open a db file. */
 
 #if defined(USE_DB) && defined(DB_VERSION_STRING)
 void
-#if DB_VERSION_MAJOR > 4 || (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
-dbfn_bdb_error_callback(const DB_ENV *dbenv, const char *pfx, const char *msg)
-{
-dbenv = dbenv;
-#else
 dbfn_bdb_error_callback(const char *pfx, char *msg)
 {
-#endif
 pfx = pfx;
 printf("Berkeley DB error: %s\n", msg);
 }
@@ -151,8 +143,8 @@ EXIM_DB *d;
 EXIM_DATUM key, content;
 uschar *bptr;
 uschar  keybuffer[256];
-uschar  temp_dbmname[512];
-uschar  real_dbmname[512];
+uschar  temp_dbmname[256];
+uschar  real_dbmname[256];
 uschar *buffer = malloc(max_outsize);
 uschar *line = malloc(max_insize);
 
@@ -195,15 +187,6 @@ if (Ustrcmp(argv[arg], argv[arg+1]) == 0)
   }
 #endif
 
-/* Check length of filename; allow for adding .dbmbuild_temp and .db or
-.dir/.pag later. */
-
-if (strlen(argv[arg+1]) > sizeof(temp_dbmname) - 20)
-  {
-  printf("exim_dbmbuild: output filename is ridiculously long\n");
-  exit(1);
-  }
-
 Ustrcpy(temp_dbmname, argv[arg+1]);
 Ustrcat(temp_dbmname, ".dbmbuild_temp");
 
@@ -216,7 +199,7 @@ if (d == NULL)
   {
   printf("exim_dbmbuild: unable to create %s: %s\n", temp_dbmname,
     strerror(errno));
-  (void)fclose(f);
+  fclose(f);
   exit(1);
   }
 
@@ -243,8 +226,7 @@ while (Ufgets(line, max_insize, f) != NULL)
   if (len >= max_insize - 1 && p[-1] != '\n')
     {
     printf("Overlong line read: max permitted length is %d\n", max_insize - 1);
-    yield = 2;
-    goto TIDYUP;
+    return 1;
     }
 
   if (line[0] == '#') continue;
@@ -270,8 +252,7 @@ while (Ufgets(line, max_insize, f) != NULL)
       {
       printf("Continued set of lines is too long: max permitted length is %d\n",
         max_outsize -1);
-      yield = 2;
-      goto TIDYUP;
+      return 1;
       }
 
     Ustrcpy(bptr, s);
@@ -415,7 +396,7 @@ if (started)
 TIDYUP:
 
 EXIM_DBCLOSE(d);
-(void)fclose(f);
+fclose(f);
 
 /* If successful, output the number of entries and rename the temporary
 files. */
