@@ -1,10 +1,8 @@
-/* $Cambridge: exim/exim-src/src/exim_dbutil.c,v 1.12 2009/11/16 19:50:36 nm4 Exp $ */
-
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2004 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -63,11 +61,10 @@ not too much extra baggage. */
 
 /* Identifiers for the different database types. */
 
-#define type_retry     1
-#define type_wait      2
-#define type_misc      3
-#define type_callout   4
-#define type_ratelimit 5
+#define type_retry   1
+#define type_wait    2
+#define type_misc    3
+#define type_callout 4
 
 
 
@@ -78,18 +75,12 @@ not too much extra baggage. */
 
 /* For Berkeley DB >= 2, we can define a function to be called in case of DB
 errors. This should help with debugging strange DB problems, e.g. getting "File
-exists" when you try to open a db file. The API changed at release 4.3. */
+exists" when you try to open a db file. */
 
 #if defined(USE_DB) && defined(DB_VERSION_STRING)
 void
-#if DB_VERSION_MAJOR > 4 || (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
-dbfn_bdb_error_callback(const DB_ENV *dbenv, const char *pfx, const char *msg)
-{
-dbenv = dbenv;
-#else
 dbfn_bdb_error_callback(const char *pfx, char *msg)
 {
-#endif
 pfx = pfx;
 printf("Berkeley DB error: %s\n", msg);
 }
@@ -120,7 +111,7 @@ static void
 usage(uschar *name, uschar *options)
 {
 printf("Usage: exim_%s%s  <spool-directory> <database-name>\n", name, options);
-printf("  <database-name> = retry | misc | wait-<transport-name> | callout | ratelimit\n");
+printf("       <database-name> = retry | misc | wait-<transport-name> | callout\n");
 exit(1);
 }
 
@@ -142,7 +133,6 @@ if (argc == 3)
   if (Ustrcmp(argv[2], "misc") == 0) return type_misc;
   if (Ustrncmp(argv[2], "wait-", 5) == 0) return type_wait;
   if (Ustrcmp(argv[2], "callout") == 0) return type_callout;
-  if (Ustrcmp(argv[2], "ratelimit") == 0) return type_ratelimit;
   }
 usage(name, options);
 return -1;              /* Never obeyed */
@@ -316,7 +306,7 @@ if (rc < 0)
   printf("** Failed to get %s lock for %s: %s",
     ((flags & O_RDONLY) != 0)? "read" : "write", buffer,
     (errno == ETIMEDOUT)? "timed out" : strerror(errno));
-  (void)close(dbblock->lockfd);
+  close(dbblock->lockfd);
   return NULL;
   }
 
@@ -336,7 +326,7 @@ if (dbblock->dbptr == NULL)
     ""
     #endif
     );
-  (void)close(dbblock->lockfd);
+  close(dbblock->lockfd);
   return NULL;
   }
 
@@ -361,7 +351,7 @@ static void
 dbfn_close(open_db *dbblock)
 {
 EXIM_DBCLOSE(dbblock->dbptr);
-(void)close(dbblock->lockfd);
+close(dbblock->lockfd);
 }
 
 
@@ -544,7 +534,6 @@ while (key != NULL)
   dbdata_retry *retry;
   dbdata_wait *wait;
   dbdata_callout_cache *callout;
-  dbdata_ratelimit *ratelimit;
   int count_bad = 0;
   int i, length;
   uschar *t;
@@ -670,15 +659,6 @@ while (key != NULL)
           print_cache(callout->random_result));
         }
 
-      break;
-
-      case type_ratelimit:
-      ratelimit = (dbdata_ratelimit *)value;
-
-      printf("%s.%06d rate: %10.3f key: %s\n",
-        print_time(ratelimit->time_stamp), ratelimit->time_usec,
-        ratelimit->rate, keybuffer);
-
       break;
       }
     store_reset(value);
@@ -751,7 +731,6 @@ for(;;)
   dbdata_retry *retry;
   dbdata_wait *wait;
   dbdata_callout_cache *callout;
-  dbdata_ratelimit *ratelimit;
   int i, oldlength;
   uschar *t;
   uschar field[256], value[256];
@@ -767,20 +746,20 @@ for(;;)
   /* If the buffer contains just one digit, or just consists of "d", use the
   previous name for an update. */
 
-  if ((isdigit((uschar)buffer[0]) && (buffer[1] == ' ' || buffer[1] == '\0'))
-       || Ustrcmp(buffer, "d") == 0)
+  if ((isdigit((uschar)buffer[0]) && !isdigit((uschar)buffer[1])) ||
+       Ustrcmp(buffer, "d") == 0)
     {
     if (name[0] == 0)
       {
       printf("No previous record name is set\n");
       continue;
       }
-    (void)sscanf(CS buffer, "%s %s", field, value);
+    sscanf(CS buffer, "%s %s", field, value);
     }
   else
     {
     name[0] = 0;
-    (void)sscanf(CS buffer, "%s %s %s", name, field, value);
+    sscanf(CS buffer, "%s %s %s", name, field, value);
     }
 
   /* Handle an update request */
@@ -892,31 +871,6 @@ for(;;)
               break;
               }
             break;
-
-            case type_ratelimit:
-            ratelimit = (dbdata_ratelimit *)record;
-            length = sizeof(dbdata_ratelimit);
-            switch(fieldno)
-              {
-              case 0:
-              if ((tt = read_time(value)) > 0) ratelimit->time_stamp = tt;
-                else printf("bad time value\n");
-              break;
-
-              case 1:
-              ratelimit->time_usec = Uatoi(value);
-              break;
-
-              case 2:
-              ratelimit->rate = Ustrtod(value, NULL);
-              break;
-
-              default:
-              printf("unknown field number\n");
-              verify = 0;
-              break;
-              }
-            break;
             }
 
           dbfn_write(dbm, name, record, length);
@@ -1014,13 +968,6 @@ for(;;)
             callout->random_result);
         }
       break;
-
-      case type_ratelimit:
-      ratelimit = (dbdata_ratelimit *)record;
-      printf("0 time stamp:  %s\n", print_time(ratelimit->time_stamp));
-      printf("1 fract. time: .%06d\n", ratelimit->time_usec);
-      printf("2 sender rate: % .3f\n", ratelimit->rate);
-      break;
       }
     }
 
