@@ -92,6 +92,8 @@ them. Also, the tls_ variables are now always visible. */
 BOOL    tls_active             = -1;
 BOOL    tls_certificate_verified = FALSE;
 uschar *tls_cipher             = NULL;
+BOOL    tls_on_connect         = FALSE;
+uschar *tls_on_connect_ports   = NULL;
 uschar *tls_peerdn             = NULL;
 
 #ifdef SUPPORT_TLS
@@ -101,7 +103,6 @@ uschar *tls_certificate        = NULL;
 uschar *tls_crl                = NULL;
 uschar *tls_dhparam            = NULL;
 BOOL    tls_offered            = FALSE;
-BOOL    tls_on_connect         = FALSE;
 uschar *tls_privatekey         = NULL;
 BOOL    tls_remember_esmtp     = FALSE;
 uschar *tls_require_ciphers    = NULL;
@@ -161,6 +162,8 @@ uschar *acl_smtp_expn          = NULL;
 uschar *acl_smtp_helo          = NULL;
 uschar *acl_smtp_mail          = NULL;
 uschar *acl_smtp_mailauth      = NULL;
+uschar *acl_smtp_predata       = NULL;
+uschar *acl_smtp_quit          = NULL;
 uschar *acl_smtp_rcpt          = NULL;
 uschar *acl_smtp_starttls      = NULL;
 uschar *acl_smtp_vrfy          = NULL;
@@ -173,31 +176,37 @@ string_item *acl_warn_logged   = NULL;
 /* Names of SMTP places for use in ACL error messages, and corresponding SMTP
 error codes - keep in step with definitions of ACL_WHERE_xxxx in macros.h. */
 
-uschar *acl_wherenames[]       = { US"AUTH",
-                                   US"connection",
+uschar *acl_wherenames[]       = { US"RCPT",
+                                   US"MAIL",
+                                   US"PREDATA",
                                    US"DATA",
+                                   US"non-SMTP",
+                                   US"AUTH",
+                                   US"connection",
                                    US"ETRN",
                                    US"EXPN",
                                    US"EHLO or HELO",
-                                   US"MAIL",
                                    US"MAILAUTH",
-                                   US"RCPT",
+                                   US"QUIT",
                                    US"STARTTLS",
-                                   US"VRFY",
-                                   US"non-SMTP" };
+                                   US"VRFY"
+                                 };
 
-int     acl_wherecodes[]       = { 503,     /* AUTH */
-                                   550,     /* connect */
+int     acl_wherecodes[]       = { 550,     /* RCPT */
+                                   550,     /* MAIL */
+                                   550,     /* PREDATA */
                                    550,     /* DATA */
+                                   0,       /* not SMTP; not relevant */
+                                   503,     /* AUTH */
+                                   550,     /* connect */
                                    458,     /* ETRN */
                                    550,     /* EXPN */
                                    550,     /* HELO/EHLO */
-                                   550,     /* MAIL */
                                    0,       /* MAILAUTH; not relevant */
-                                   550,     /* RCPT */
+                                   0,       /* QUIT; not relevant */
                                    550,     /* STARTTLS */
-                                   252,     /* VRFY */
-                                   0 };     /* not SMTP; not relevant */
+                                   252      /* VRFY */
+                                 };
 
 BOOL    accept_8bitmime        = FALSE;
 address_item  *addr_duplicate  = NULL;
@@ -518,6 +527,7 @@ uschar *host_reject_connection = NULL;
 tree_node *hostlist_anchor     = NULL;
 int     hostlist_count         = 0;
 uschar *hosts_treat_as_local   = NULL;
+uschar *hosts_connection_nolog = NULL;
 
 int     ignore_bounce_errors_after = 10*7*24*60*60;  /* 10 weeks */
 BOOL    ignore_fromline_local  = FALSE;
@@ -635,8 +645,10 @@ int     message_size           = 0;
 uschar *message_size_limit     = US"50M";
 uschar  message_subdir[2]      = { 0, 0 };
 uschar *message_reference      = NULL;
+BOOL    mua_wrapper            = FALSE;
 
 uid_t  *never_users            = NULL;
+BOOL    no_multiline_responses = FALSE;
 
 uid_t   original_euid;
 gid_t   originator_gid;
@@ -840,6 +852,7 @@ BOOL    search_find_defer      = FALSE;
 uschar *self_hostname          = NULL;
 uschar *sender_address         = NULL;
 unsigned int sender_address_cache[(MAX_NAMED_LIST * 2)/32];
+uschar *sender_address_data    = NULL;
 BOOL    sender_address_forced  = FALSE;
 uschar *sender_address_unrewritten = NULL;
 uschar *sender_data            = NULL;
