@@ -88,6 +88,48 @@ return (unsigned int)(random_seed >> 16) % limit;
 
 
 
+/*************************************************
+*         Sort addresses when testing            *
+*************************************************/
+
+/* This function is called only when running in the test harness. It sorts a
+number of multihomed host IP addresses into the order, so as to get
+repeatability. This doesn't have to be efficient. But don't interchange IPv4
+and IPv6 addresses!
+
+Arguments:
+  host        -> the first host item
+  last        -> the last host item
+  
+Returns:      nothing
+*/  
+
+static void
+sort_addresses(host_item *host, host_item *last)
+{
+BOOL done = FALSE;
+while (!done)
+  {
+  host_item *h;
+  done = TRUE;
+  for (h = host; h != last; h = h->next)
+    {
+    if ((Ustrchr(h->address, ':') == NULL) !=
+        (Ustrchr(h->next->address, ':') == NULL))
+      continue;
+    if (Ustrcmp(h->address, h->next->address) > 0)
+      {
+      uschar *temp = h->address;
+      h->address = h->next->address;
+      h->next->address = temp;
+      done = FALSE;
+      }
+    }
+  }
+}
+
+
+
 /*************************************************
 *       Build chain of host items from list      *
 *************************************************/
@@ -710,12 +752,18 @@ if (Ustrchr(address, ':') != NULL)
 
   if (*p == ':') p++;
 
-  /* Split the address into components separated by colons. */
+  /* Split the address into components separated by colons. The input address 
+  is supposed to be checked for syntax. There was a case where this was 
+  overlooked; to guard against that happening again, check here and crash if 
+  there is a violation. */
 
   while (*p != 0)
     {
     int len = Ustrcspn(p, ":");
     if (len == 0) nulloffset = ci;
+    if (ci > 7) log_write(0, LOG_MAIN|LOG_PANIC_DIE, 
+      "Internal error: invalid IPv6 address \"%s\" passed to host_aton()",
+      address);  
     component[ci++] = p;
     p += len;
     if (*p == ':') p++;
@@ -812,22 +860,24 @@ for (i = 0; i < count; i++)
 /* We can't use host_ntoa() because it assumes the binary values are in network
 byte order, and these are the result of host_aton(), which puts them in ints in
 host byte order. Also, we really want IPv6 addresses to be in a canonical
-format, so we output them with no abbreviation. However, we can't use the
-normal colon separator in them because it terminates keys in lsearch files, so
-use dot instead.
+format, so we output them with no abbreviation. In a number of cases we can't
+use the normal colon separator in them because it terminates keys in lsearch
+files, so we want to use dot instead. There's an argument that specifies what 
+to use for IPv6 addresses.
 
 Arguments:
   count       1 or 4 (number of ints)
   binary      points to the ints
   mask        mask value; if < 0 don't add to result
   buffer      big enough to hold the result
+  sep         component separator character for IPv6 addresses 
 
 Returns:      the number of characters placed in buffer, not counting
               the final nul.
 */
 
 int
-host_nmtoa(int count, int *binary, int mask, uschar *buffer)
+host_nmtoa(int count, int *binary, int mask, uschar *buffer, int sep)
 {
 int i, j;
 uschar *tt = buffer;
@@ -846,12 +896,12 @@ else
   for (i = 0; i < 4; i++)
     {
     j = binary[i];
-    sprintf(CS tt, "%04x.%04x.", (j >> 16) & 0xffff, j & 0xffff);
+    sprintf(CS tt, "%04x%c%04x%c", (j >> 16) & 0xffff, sep, j & 0xffff, sep);
     while (*tt) tt++;
     }
   }
 
-tt--;   /* lose final . */
+tt--;   /* lose final separator */
 
 if (mask < 0)
   *tt = 0;
@@ -866,6 +916,43 @@ return tt - buffer;
 
 
 
+/*************************************************
+*        Check port for tls_on_connect           *
+*************************************************/
+
+/* This function checks whether a given incoming port is configured for tls-
+on-connect. It is called from the daemon and from inetd handling. If the global
+option tls_on_connect is already set, all ports operate this way. Otherwise, we
+check the tls_on_connect_ports option for a list of ports.
+
+Argument:  a port number
+Returns:   TRUE or FALSE
+*/
+
+BOOL
+host_is_tls_on_connect_port(int port)
+{
+int sep = 0;
+uschar buffer[32];
+uschar *list = tls_on_connect_ports;
+uschar *s;
+
+if (tls_on_connect) return TRUE;
+
+while ((s = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
+  {
+  uschar *end;
+  int lport = Ustrtol(s, &end, 10);
+  if (*end != 0) log_write(0, LOG_MAIN|LOG_PANIC_DIE, "tls_on_connect_ports "
+    "contains \"%s\", which is not a port number: exim abandoned", s);
+  if (lport == port) return TRUE;
+  }
+
+return FALSE;
+}
+
+
+
 /*************************************************
 *        Check whether host is in a network      *
 *************************************************/
@@ -1752,31 +1839,9 @@ yield = local_host_check?
   host_scan_for_local_hosts(host, &last, NULL) : HOST_FOUND;
 
 /* When running in the test harness, sort into the order of addresses so as to
-get repeatability. This doesn't have to be efficient. But don't interchange
-IPv4 and IPv6 addresses! */
+get repeatability. */
 
-if (running_in_test_harness)
-  {
-  BOOL done = FALSE;
-  while (!done)
-    {
-    host_item *h;
-    done = TRUE;
-    for (h = host; h != last; h = h->next)
-      {
-      if ((Ustrchr(h->address, ':') == NULL) !=
-          (Ustrchr(h->next->address, ':') == NULL))
-        continue;
-      if (Ustrcmp(h->address, h->next->address) > 0)
-        {
-        uschar *temp = h->address;
-        h->address = h->next->address;
-        h->next->address = temp;
-        done = FALSE;
-        }
-      }
-    }
-  }
+if (running_in_test_harness) sort_addresses(host, last);
 
 HDEBUG(D_host_lookup)
   {
@@ -1915,7 +1980,7 @@ for (; i >= 0; i--)
   fails or times out, but not if another one succeeds. (In the early
   IPv6 days there are name servers that always fail on AAAA, but are happy
   to give out an A record. We want to proceed with that A record.) */
-
+  
   if (rc != DNS_SUCCEED)
     {
     if (i == 0)  /* Just tried for an A record, i.e. end of loop */
@@ -2075,9 +2140,12 @@ Arguments:
                           HOST_FIND_BY_SRV  => look for SRV
                           HOST_FIND_BY_MX   => look for MX
                           HOST_FIND_BY_A    => look for A or AAAA
+                        also flags indicating how the lookup is done
+                          HOST_FIND_QUALIFY_SINGLE   ) passed to the
+                          HOST_FIND_SEARCH_PARENTS   )   resolver
   srv_service           when SRV used, the service name
-  qualify_single        if TRUE, ask DNS to qualify single-component names
-  search_parents        if TRUE, asd DNS to search parent domains
+  srv_fail_domains      DNS errors for these domains => assume nonexist
+  mx_fail_domains       DNS errors for these domains => assume nonexist
   fully_qualified_name  if not NULL, return fully-qualified name
   removed               set TRUE if local host was removed from the list
 
@@ -2094,7 +2162,7 @@ Returns:                HOST_FIND_FAILED  Failed to find the host or domain;
 
 int
 host_find_bydns(host_item *host, uschar *ignore_target_hosts, int whichrrs,
-  uschar *srv_service, BOOL qualify_single, BOOL search_parents,
+  uschar *srv_service, uschar *srv_fail_domains, uschar *mx_fail_domains,
   uschar **fully_qualified_name, BOOL *removed)
 {
 host_item *h, *last;
@@ -2110,7 +2178,8 @@ resolver if necessary, set up the relevant options, and initialize the flag
 that gets set for DNS syntax check errors. */
 
 if (fully_qualified_name != NULL) *fully_qualified_name = host->name;
-dns_init(qualify_single, search_parents);
+dns_init((whichrrs & HOST_FIND_QUALIFY_SINGLE) != 0,
+         (whichrrs & HOST_FIND_SEARCH_PARENTS) != 0);
 host_find_failed_syntax = FALSE;
 
 /* First, if requested, look for SRV records. The service name is given; we
@@ -2135,21 +2204,39 @@ if ((whichrrs & HOST_FIND_BY_SRV) != 0)
   if (temp_fully_qualified_name != buffer && fully_qualified_name != NULL)
     *fully_qualified_name = temp_fully_qualified_name + prefix_length;
 
-  if (rc == DNS_FAIL || rc == DNS_AGAIN) return HOST_FIND_AGAIN;
+  /* On DNS failures, we give the "try again" error unless the domain is
+  listed as one for which we continue. */
+
+  if (rc == DNS_FAIL || rc == DNS_AGAIN)
+    {
+    if (match_isinlist(host->name, &srv_fail_domains, 0, NULL, NULL, MCL_DOMAIN,
+        TRUE, NULL) != OK)
+      return HOST_FIND_AGAIN;
+    DEBUG(D_host_lookup) debug_printf("DNS_%s treated as DNS_NODATA "
+      "(domain in srv_fail_domains)\n", (rc == DNS_FAIL)? "FAIL":"AGAIN");
+    }
   }
 
 /* If we did not find any SRV records, search the DNS for MX records, if
 requested to do so. If the result is DNS_NOMATCH, it means there is no such
 domain, and there's no point in going on to look for address records with the
 same domain. The result will be DNS_NODATA if the domain exists but has no MX
-records. */
+records. On DNS failures, we give the "try again" error unless the domain is
+listed as one for which we continue. */
 
 if (rc != DNS_SUCCEED && (whichrrs & HOST_FIND_BY_MX) != 0)
   {
   ind_type = T_MX;
   rc = dns_lookup(&dnsa, host->name, ind_type, fully_qualified_name);
-  if (rc == DNS_FAIL || rc == DNS_AGAIN) return HOST_FIND_AGAIN;
   if (rc == DNS_NOMATCH) return HOST_FIND_FAILED;
+  if (rc == DNS_FAIL || rc == DNS_AGAIN)
+    {
+    if (match_isinlist(host->name, &mx_fail_domains, 0, NULL, NULL, MCL_DOMAIN,
+        TRUE, NULL) != OK)
+      return HOST_FIND_AGAIN;
+    DEBUG(D_host_lookup) debug_printf("DNS_%s treated as DNS_NODATA "
+      "(domain in mx_fail_domains)\n", (rc == DNS_FAIL)? "FAIL":"AGAIN");
+    }
   }
 
 /* If we haven't found anything yet, and we are requested to do so, try for an
@@ -2158,7 +2245,7 @@ host. */
 
 if (rc != DNS_SUCCEED)
   {
-  if ((whichrrs &HOST_FIND_BY_A) == 0)
+  if ((whichrrs & HOST_FIND_BY_A) == 0)
     {
     DEBUG(D_host_lookup) debug_printf("Address records are not being sought\n");
     return HOST_FIND_FAILED;
@@ -2181,6 +2268,11 @@ if (rc != DNS_SUCCEED)
   else
     if (rc == HOST_IGNORED) rc = HOST_FIND_FAILED;  /* No special action */
 
+  /* When running in the test harness, sort into the order of addresses so as
+  to get repeatability. */
+  
+  if (running_in_test_harness) sort_addresses(host, last);
+
   DEBUG(D_host_lookup)
     {
     host_item *h;
@@ -2871,6 +2963,8 @@ while (Ufgets(buffer, 256, stdin) != NULL)
     }
   else
     {
+    int flags = whichrrs;
+
     h.name = buffer;
     h.next = NULL;
     h.mx = MX_NONE;
@@ -2879,11 +2973,14 @@ while (Ufgets(buffer, 256, stdin) != NULL)
     h.why = hwhy_unknown;
     h.address = NULL;
 
+    if (qualify_single) flags |= HOST_FIND_QUALIFY_SINGLE;
+    if (search_parents) flags |= HOST_FIND_SEARCH_PARENTS;
+
     rc = byname?
       host_find_byname(&h, NULL, &fully_qualified_name, TRUE)
       :
-      host_find_bydns(&h, NULL, whichrrs, US"smtp", qualify_single,
-        search_parents, &fully_qualified_name, NULL);
+      host_find_bydns(&h, NULL, flags, US"smtp", NULL, NULL,
+        &fully_qualified_name, NULL);
 
     if (rc == HOST_FIND_FAILED) printf("Failed\n");
       else if (rc == HOST_FIND_AGAIN) printf("Again\n");
