@@ -35,7 +35,6 @@ static uschar *rejectlog_datestamp = NULL;
 
 static int    mainlogfd = -1;
 static int    rejectlogfd = -1;
-static int    processlogfd = -1;
 static ino_t  mainlog_inode = 0;
 static ino_t  rejectlog_inode = 0;
 
@@ -975,13 +974,14 @@ written to a file - never to syslog. */
 
 if ((flags & LOG_PROCESS) != 0)
   {
-  if (processlogfd < 0)
-    open_log(&processlogfd, lt_process);  /* No return on error */
+  int processlogfd;
+  open_log(&processlogfd, lt_process);  /* No return on error */
   if ((rc = write(processlogfd, log_buffer, length)) != length)
     {
     log_write_failed(US"process log", length, rc);
     /* That function does not return */
     }
+  (void)close(processlogfd);
   }
 
 
@@ -1043,8 +1043,6 @@ log_close_all(void)
 {
 if (mainlogfd >= 0)
   { close(mainlogfd); mainlogfd = -1; }
-if (processlogfd >= 0)
-  { close(processlogfd); processlogfd = -1; }
 if (rejectlogfd >= 0)
   { close(rejectlogfd); rejectlogfd = -1; }
 closelog();
