@@ -70,7 +70,7 @@ as unsigned. */
 a no-op once an SSL session is in progress. */
 
 #ifdef SUPPORT_TLS
-#define mac_smtp_fflush() if (!tls_active) fflush(smtp_out);
+#define mac_smtp_fflush() if (tls_active < 0) fflush(smtp_out);
 #else
 #define mac_smtp_fflush() fflush(smtp_out);
 #endif
@@ -129,12 +129,12 @@ enough to hold all the headers from a normal kind of message. */
 into big_buffer_size and in some circumstances increased. It should be at least
 as long as the maximum path length. */
 
-#if defined PATH_MAX && PATH_MAX > 1024
+#if defined PATH_MAX && PATH_MAX > 16384
 #define BIG_BUFFER_SIZE PATH_MAX
-#elif defined MAXPATHLEN && MAXPATHLEN > 1024
+#elif defined MAXPATHLEN && MAXPATHLEN > 16384
 #define BIG_BUFFER_SIZE MAXPATHLEN
 #else
-#define BIG_BUFFER_SIZE 1024
+#define BIG_BUFFER_SIZE 16384
 #endif
 
 /* This limits the length of data returned by local_scan(). Because it is
@@ -251,6 +251,13 @@ local_scan() function. Do not change them unilaterally. */
 #define FAIL_SEND     13    /* send() failed in authenticator */
 #define FAIL_DROP     14    /* Fail and drop connection (used in ACL) */
 
+/* Returns from the deliver_message() function */
+
+#define DELIVER_ATTEMPTED_NORMAL   0  /* Tried a normal delivery */
+#define DELIVER_MUA_SUCCEEDED      1  /* Success when mua_wrapper is set */
+#define DELIVER_MUA_FAILED         2  /* Failure when mua_wrapper is set */
+#define DELIVER_NOT_ATTEMPTED      3  /* Not tried (no msg or is locked */
+
 /* Returns from DNS lookup functions. */
 
 enum { DNS_SUCCEED, DNS_NOMATCH, DNS_NODATA, DNS_AGAIN, DNS_FAIL };
@@ -423,6 +430,7 @@ to conflict with system errno values. */
 #define ERRNO_CALLOUTDEFER   (-41)   /* When verifying */
 #define ERRNO_AUTHFAIL       (-42)   /* When required by client */
 #define ERRNO_CONNECTTIMEOUT (-43)   /* Used internally in smtp transport */
+#define ERRNO_RCPT4XX        (-44)   /* RCPT gave 4xx error */
 
 /* These must be last, so all retry deferments can easily be identified */
 
@@ -526,6 +534,14 @@ return values from header_checkname(). */
 #define htype_envelope_to   'e'
 #define htype_subject       's'
 
+/* These values are used only when adding new headers from an ACL; they too
+never get onto the spool. The type of the added header is set by reference
+to the header name, by calling header_checkname(). */
+
+#define htype_add_top       'a'
+#define htype_add_rec       'r'
+#define htype_add_bot       'z'
+
 /* Types of item in options lists. These are the bottom 8 bits of the "type"
 field, which is an int. The opt_void value is used for entries in tables that
 point to special types of value that are accessed only indirectly (e.g. the
@@ -656,9 +672,11 @@ enum {
 
 /* Flags for host_find_bydns() */
 
-#define HOST_FIND_BY_SRV       0x001
-#define HOST_FIND_BY_MX        0x002
-#define HOST_FIND_BY_A         0x004
+#define HOST_FIND_BY_SRV          0x0001
+#define HOST_FIND_BY_MX           0x0002
+#define HOST_FIND_BY_A            0x0004
+#define HOST_FIND_QUALIFY_SINGLE  0x0008
+#define HOST_FIND_SEARCH_PARENTS  0x0010
 
 /* Actions applied to specific messages. */
 
@@ -706,13 +724,28 @@ be added */
 
 enum { MCL_STRING, MCL_DOMAIN, MCL_HOST, MCL_ADDRESS, MCL_LOCALPART };
 
-/* Codes for the places from which ACLs can be called. Keep in step with the
-tables of names and response codes in globals.c. */
+/* Codes for the places from which ACLs can be called. These are cunningly
+ordered to make it easy to implement tests for certain ACLs when processing
+"control" modifiers, by means of a maximum "where" value. Do not modify this
+order without checking carefully! Furthermore, remember to keep these in step
+with the tables of names and response codes in globals.c. */
 
-enum { ACL_WHERE_AUTH,     ACL_WHERE_CONNECT, ACL_WHERE_DATA, ACL_WHERE_ETRN,
-       ACL_WHERE_EXPN,     ACL_WHERE_HELO,    ACL_WHERE_MAIL,
-       ACL_WHERE_MAILAUTH, ACL_WHERE_RCPT,
-       ACL_WHERE_STARTTLS, ACL_WHERE_VRFY,    ACL_WHERE_NOTSMTP };
+enum { ACL_WHERE_RCPT,       /* Some controls are for RCPT only */
+       ACL_WHERE_MAIL,       /* )                                           */
+       ACL_WHERE_PREDATA,    /* ) There are several tests for "in message", */
+       ACL_WHERE_DATA,       /* ) implemented by <= WHERE_NOTSMTP           */
+       ACL_WHERE_NOTSMTP,    /* )                                           */
+
+       ACL_WHERE_AUTH,       /* These remaining ones are not currently    */
+       ACL_WHERE_CONNECT,    /* required to be in a special order so they */
+       ACL_WHERE_ETRN,       /* are just alphabetical.                    */
+       ACL_WHERE_EXPN,
+       ACL_WHERE_HELO,
+       ACL_WHERE_MAILAUTH,
+       ACL_WHERE_QUIT,
+       ACL_WHERE_STARTTLS,
+       ACL_WHERE_VRFY
+     };
 
 /* Situations for spool_write_header() */
 
