@@ -210,8 +210,8 @@ if (cb->at_is_special && pattern[0] == '@')
       ignore_target_hosts,
       HOST_FIND_BY_MX,     /* search only for MX, not SRV or A */
       NULL,                /* service name not relevant */
-      FALSE,               /* no widening */
-      FALSE,               /* no widening */
+      NULL,                /* srv_fail_domains not relevant */
+      NULL,                /* mx_fail_domains not relevant */
       NULL,                /* no feedback FQDN */
       &removed);           /* feedback if local removed */
 
@@ -438,12 +438,15 @@ HDEBUG(D_any)
   if (listname[0] != 0) ot = string_sprintf("%s in %s?", name, listname);
   }
 
-/* If the list is empty, the answer is no. This case happens only when checking
-named options, so ot will be set. */
+/* If the list is empty, the answer is no. Skip the debugging output for
+an unnamed list. */
 
 if (*listptr == NULL)
   {
-  HDEBUG(D_lists) debug_printf("%s no (option unset)\n", ot);
+  HDEBUG(D_lists)
+    {
+    if (ot != NULL) debug_printf("%s no (option unset)\n", ot);
+    }
   return FAIL;
   }
 
@@ -625,9 +628,11 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
             store_pool = POOL_PERM;
             p = store_get(sizeof(namedlist_cacheblock));
             p->key = string_copy(get_check_key(arg, type));
+
+
+            p->data = (*valueptr == NULL)? NULL : string_copy(*valueptr);
             store_pool = old_pool;
 
-            p->data = *valueptr;
             p->next = nb->cache_data;
             nb->cache_data = p;
             if (*valueptr != NULL)
@@ -848,6 +853,8 @@ Arguments:
                  MCL_LOCALPART when matching a local part list (address lists
                    have their own function)
                  MCL_STRING for others (e.g. list of ciphers)
+                 MCL_NOEXPAND (whose value is greater than any of them) may
+                   be added to any value to suppress expansion of the list
   caseless       TRUE for (mostly) caseless matching - passed directly to
                    match_check_string()
   valueptr       pointer to where any lookup data is to be passed back,
@@ -872,7 +879,7 @@ cb.subject = caseless? string_copylc(s) : string_copy(s);
 cb.expand_setup = (sep > UCHAR_MAX)? 0 : -1;
 cb.use_partial = TRUE;
 cb.caseless = caseless;
-cb.at_is_special = (type == MCL_DOMAIN);
+cb.at_is_special = (type == MCL_DOMAIN || type == MCL_DOMAIN + MCL_NOEXPAND);
 if (valueptr != NULL) *valueptr = NULL;
 return  match_check_list(listptr, sep, anchorptr, &local_cache_bits,
   check_string, &cb, type, s, valueptr);
@@ -925,32 +932,24 @@ error = error;  /* Keep clever compilers from complaining */
 DEBUG(D_lists) debug_printf("address match: subject=%s pattern=%s\n",
   subject, pattern);
 
-/* Handle a regular expression, which must match the entire incoming address. */
+/* Handle a regular expression, which must match the entire incoming address. 
+This may be the empty address. */
 
 if (*pattern == '^')
   return match_check_string(subject, pattern, cb->expand_setup, TRUE,
     cb->caseless, FALSE, NULL);
 
-/* If the subject is the empty string, the only pattern it can match (other
-than a regular expression) is the empty pattern. */
-
-if (*subject == 0) return (*pattern == 0)? OK : FAIL;
-
-/* Find the domain in the subject */
-
-sdomain = Ustrrchr(subject, '@');
-
-/* Handle the case of a pattern that is just a lookup. Skip over possible
-lookup names (letters, digits, hyphens). Skip over a possible * or *@ at
-the end. Then we must have a semicolon for it to be a lookup. */
+/* Handle a pattern that is just a lookup. Skip over possible lookup names
+(letters, digits, hyphens). Skip over a possible * or *@ at the end. Then we
+must have a semicolon for it to be a lookup. */
 
 for (s = pattern; isalnum(*s) || *s == '-'; s++);
 if (*s == '*') s++;
 if (*s == '@') s++;
 
-/* If it is a straight lookup, do a lookup for the whole address. Partial
-matching doesn't make sense here, so we ignore it, but write a panic log entry.
-However, *@ matching will be honoured. */
+/* If it is a straight lookup, do a lookup for the whole address. This may be 
+the empty address. Partial matching doesn't make sense here, so we ignore it,
+but write a panic log entry. However, *@ matching will be honoured. */
 
 if (*s == ';')
   {
@@ -961,6 +960,16 @@ if (*s == ';')
     valueptr);
   }
 
+/* For the remaining cases, an empty subject matches only an empty pattern, 
+because other patterns expect to have a local part and a domain to match 
+against. */
+
+if (*subject == 0) return (*pattern == 0)? OK : FAIL;
+
+/* Find the subject's domain */
+
+sdomain = Ustrrchr(subject, '@');
+
 /* If the pattern starts with "@@" we have a split lookup, where the domain is
 looked up to obtain a list of local parts. If the subject's local part is just
 "*" (called from retry) the match always fails. */
