@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) University of Cambridge 1995 - 2004 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for parsing addresses */
@@ -187,7 +187,7 @@ The start of the last potential comment position is remembered to
 make it possible to ignore comments at the end of compound items.
 
 Argument: current character pointer
-Returns:  new character pointer
+Regurns:  new character pointer
 */
 
 static uschar *
@@ -241,17 +241,18 @@ s = skip_comment(s);
 any character except [ ] \, including linear white space, and may contain
 quoted characters. However, RFC 821 restricts literals to being dot-separated
 3-digit numbers, and we make the obvious extension for IPv6. Go for a sequence
-of digits, dots, hex digits, and colons here; later this will be checked for
-being a syntactically valid IP address if it ever gets to a router.
+of digits and dots (hex digits and colons for IPv6) here; later this will be
+checked for being a syntactically valid IP address if it ever gets to a router.
 
-Allow both the formal IPv6 form, with IPV6: at the start, and the informal form
-without it, and accept IPV4: as well, 'cause someone will use it sooner or
-later. */
+If IPv6 is supported, allow both the formal form, with IPV6: at the start, and
+the informal form without it, and accept IPV4: as well, 'cause someone will use
+it sooner or later. */
 
 if (*s == '[')
   {
   *t++ = *s++;
 
+  #if HAVE_IPV6
   if (strncmpic(s, US"IPv6:", 5) == 0 || strncmpic(s, US"IPv4:", 5) == 0)
     {
     memcpy(t, s, 5);
@@ -260,6 +261,10 @@ if (*s == '[')
     }
   while (*s == '.' || *s == ':' || isxdigit(*s)) *t++ = *s++;
 
+  #else
+  while (*s == '.' || isdigit(*s)) *t++ = *s++;
+  #endif
+
   if (*s == ']') *t++ = *s++; else
     {
     *errorptr = US"malformed domain literal";
@@ -421,10 +426,10 @@ for (;;)
   if (*s == '\"')
     {
     *t++ = '\"';
-    while ((c = *++s) && c != '\"')
+    while ((c = *(++s)) != 0 && c != '\"')
       {
       *t++ = c;
-      if (c == '\\' && s[1]) *t++ = *++s;
+      if (c == '\\' && s[1] != 0) *t++ = *(++s);
       }
     if (c == '\"')
       {
@@ -443,7 +448,7 @@ for (;;)
   else while (!mac_iscntrl_or_special(*s) || *s == '\\')
     {
     c = *t++ = *s++;
-    if (c == '\\' && *s) *t++ = *s++;
+    if (c == '\\' && *s != 0) *t++ = *s++;
     }
 
   /* Terminate the word and skip subsequent comment */
@@ -550,7 +555,9 @@ read_addr_spec(uschar *s, uschar *t, int term, uschar **errorptr,
 {
 s = read_local_part(s, t, errorptr, FALSE);
 if (*errorptr == NULL)
+  {
   if (*s != term)
+    {
     if (*s != '@')
       *errorptr = string_sprintf("\"@\" or \".\" expected after \"%s\"", t);
     else
@@ -560,6 +567,8 @@ if (*errorptr == NULL)
       *domainptr = t;
       s = read_domain(s, t, errorptr);
       }
+    }
+  }
 return s;
 }
 
@@ -591,15 +600,10 @@ which may appear in certain headers. If the flag parse_allow_group is set
 TRUE and parse_found_group is FALSE when this function is called, an address
 which is the start of a group (i.e. preceded by a phrase and a colon) is
 recognized; the phrase is ignored and the flag parse_found_group is set. If
-this flag is TRUE at the end of an address, and if an extraneous semicolon is
-found, it is ignored and the flag is cleared.
-
-This logic is used only when scanning through addresses in headers, either to
-fulfil the -t option, or for rewriting, or for checking header syntax. Because
-the group "state" has to be remembered between multiple calls of this function,
-the variables parse_{allow,found}_group are global. It is important to ensure
-that they are reset to FALSE at the end of scanning a header's list of
-addresses.
+this flag is TRUE at the end of an address, then if an extraneous semicolon is
+found, it is ignored and the flag is cleared. This logic is used only when
+scanning through addresses in headers, either to fulfil the -t option or for
+rewriting or checking header syntax.
 
 Arguments:
   mailbox     points to the RFC822 mailbox
@@ -620,8 +624,8 @@ parse_extract_address(uschar *mailbox, uschar **errorptr, int *start, int *end,
 {
 uschar *yield = store_get(Ustrlen(mailbox) + 1);
 uschar *startptr, *endptr;
-uschar *s = US mailbox;
-uschar *t = US yield;
+uschar *s = (uschar *)mailbox;
+uschar *t = (uschar *)yield;
 
 *domain = 0;
 
@@ -638,7 +642,7 @@ RESTART:   /* Come back here after passing a group name */
 s = skip_comment(s);
 startptr = s;                                 /* In case addr-spec */
 s = read_local_part(s, t, errorptr, TRUE);    /* Dot separated words */
-if (*errorptr) goto PARSE_FAILED;
+if (*errorptr != NULL) goto PARSE_FAILED;
 
 /* If the terminator is neither < nor @ then the format of the address
 must either be a bare local-part (we are now at the end), or a phrase
@@ -658,10 +662,10 @@ if (*s != '@' && *s != '<')
   end of string will produce a null local_part and therefore fail. We don't
   need to keep updating t, as the phrase isn't to be kept. */
 
-  while (*s != '<' && (!f.parse_allow_group || *s != ':'))
+  while (*s != '<' && (!parse_allow_group || *s != ':'))
     {
     s = read_local_part(s, t, errorptr, FALSE);
-    if (*errorptr)
+    if (*errorptr != NULL)
       {
       *errorptr = string_sprintf("%s (expected word or \"<\")", *errorptr);
       goto PARSE_FAILED;
@@ -670,8 +674,8 @@ if (*s != '@' && *s != '<')
 
   if (*s == ':')
     {
-    f.parse_found_group = TRUE;
-    f.parse_allow_group = FALSE;
+    parse_found_group = TRUE;
+    parse_allow_group = FALSE;
     s++;
     goto RESTART;
     }
@@ -686,8 +690,8 @@ processing it. Note that this is "if" rather than "else if" because it's also
 used after reading a preceding phrase.
 
 There are a lot of broken sendmails out there that put additional pairs of <>
-round <route-addr>s.  If strip_excess_angle_brackets is set, allow a limited
-number of them, as long as they match. */
+round <route-addr>s. If strip_excess_angle_brackets is set, allow any number of
+them, as long as they match. */
 
 if (*s == '<')
   {
@@ -696,11 +700,8 @@ if (*s == '<')
   int bracket_count = 1;
 
   s++;
-  if (strip_excess_angle_brackets) while (*s == '<')
-   {
-   if(bracket_count++ > 5) FAILED(US"angle-brackets nested too deep");
-   s++;
-   }
+  if (strip_excess_angle_brackets)
+    while (*s == '<') { bracket_count++; s++; }
 
   t = yield;
   startptr = s;
@@ -714,7 +715,7 @@ if (*s == '<')
   if (*s == '@')
     {
     s = read_route(s, t, errorptr);
-    if (*errorptr) goto PARSE_FAILED;
+    if (*errorptr != NULL) goto PARSE_FAILED;
     *t = 0;                  /* Ensure route is ignored - probably overkill */
     source_routed = TRUE;
     }
@@ -732,7 +733,7 @@ if (*s == '<')
   else
     {
     s = read_addr_spec(s, t, '>', errorptr, &domainptr);
-    if (*errorptr) goto PARSE_FAILED;
+    if (*errorptr != NULL) goto PARSE_FAILED;
     *domain = domainptr - yield;
     if (source_routed && *domain == 0)
       FAILED(US"domain missing in source-routed address");
@@ -742,10 +743,9 @@ if (*s == '<')
   if (*errorptr != NULL) goto PARSE_FAILED;
   while (bracket_count-- > 0) if (*s++ != '>')
     {
-    *errorptr = s[-1] == 0
-      ? US"'>' missing at end of address"
-      : string_sprintf("malformed address: %.32s may not follow %.*s",
-	  s-1, (int)(s - US mailbox - 1), mailbox);
+    *errorptr = (s[-1] == 0)? US"'>' missing at end of address" :
+      string_sprintf("malformed address: %.32s may not follow %.*s",
+        s-1, s - (uschar *)mailbox - 1, mailbox);
     goto PARSE_FAILED;
     }
 
@@ -790,21 +790,21 @@ move it back past white space if necessary. */
 PARSE_SUCCEEDED:
 if (*s != 0)
   {
-  if (f.parse_found_group && *s == ';')
+  if (parse_found_group && *s == ';')
     {
-    f.parse_found_group = FALSE;
-    f.parse_allow_group = TRUE;
+    parse_found_group = FALSE;
+    parse_allow_group = TRUE;
     }
   else
     {
     *errorptr = string_sprintf("malformed address: %.32s may not follow %.*s",
-      s, (int)(s - US mailbox), mailbox);
+      s, s - (uschar *)mailbox, mailbox);
     goto PARSE_FAILED;
     }
   }
-*start = startptr - US mailbox;      /* Return offsets */
+*start = startptr - (uschar *)mailbox;      /* Return offsets */
 while (isspace(endptr[-1])) endptr--;
-*end = endptr - US mailbox;
+*end = endptr - (uschar *)mailbox;
 
 /* Although this code has no limitation on the length of address extracted,
 other parts of Exim may have limits, and in any case, RFC 2821 limits local
@@ -817,17 +817,17 @@ if (*end - *start > ADDRESS_MAXLENGTH)
   return NULL;
   }
 
-return yield;
+return (uschar *)yield;
 
 /* Use goto (via the macro FAILED) to get to here from a variety of places.
 We might have an empty address in a group - the caller can choose to ignore
 this. We must, however, keep the flags correct. */
 
 PARSE_FAILED:
-if (f.parse_found_group && *s == ';')
+if (parse_found_group && *s == ';')
   {
-  f.parse_found_group = FALSE;
-  f.parse_allow_group = TRUE;
+  parse_found_group = FALSE;
+  parse_allow_group = TRUE;
   }
 return NULL;
 }
@@ -845,9 +845,9 @@ If the only characters that strictly need quoting are spaces, we return the
 original string, unmodified. If a quoted string is too long for the buffer, it
 is truncated. (This shouldn't happen: this is normally handling short strings.)
 
-Hmmph. As always, things get perverted for other uses. This function was
-originally for the "phrase" part of addresses. Now it is being used for much
-longer texts in ACLs and via the ${rfc2047: expansion item. This means we have
+Hmmph. As always, things get perverted for other uses. This function was 
+originally for the "phrase" part of addresses. Now it is being used for much 
+longer texts in ACLs and via the ${rfc2047: expansion item. This means we have 
 to check for overlong "encoded-word"s and split them. November 2004.
 
 Arguments:
@@ -857,8 +857,6 @@ Arguments:
   charset      the name of the character set; NULL => iso-8859-1
   buffer       the buffer to put the answer in
   buffer_size  the size of the buffer
-  fold         if TRUE, a newline is inserted before the separating space when
-                 more than one encoded-word is generated
 
 Returns:       pointer to the original string, if no quoting needed, or
                pointer to buffer containing the quoted string, or
@@ -866,17 +864,16 @@ Returns:       pointer to the original string, if no quoting needed, or
                the introduction
 */
 
-const uschar *
-parse_quote_2047(const uschar *string, int len, uschar *charset, uschar *buffer,
-  int buffer_size, BOOL fold)
+uschar *
+parse_quote_2047(uschar *string, int len, uschar *charset, uschar *buffer,
+  int buffer_size)
 {
-const uschar *s = string;
+uschar *s = string;
 uschar *p, *t;
 int hlen;
 BOOL coded = FALSE;
-BOOL first_byte = FALSE;
 
-if (!charset) charset = US"iso-8859-1";
+if (charset == NULL) charset = US"iso-8859-1";
 
 /* We don't expect this to fail! */
 
@@ -891,41 +888,35 @@ for (; len > 0; len--)
   {
   int ch = *s++;
   if (t > buffer + buffer_size - hlen - 8) break;
-
-  if ((t - p > 67) && !first_byte)
+  
+  if (t - p > 70)
     {
     *t++ = '?';
     *t++ = '=';
-    if (fold) *t++ = '\n';
     *t++ = ' ';
     p = t;
     Ustrncpy(p, buffer, hlen);
     t += hlen;
-    }
-
+    }      
+ 
   if (ch < 33 || ch > 126 ||
       Ustrchr("?=()<>@,;:\\\".[]_", ch) != NULL)
     {
-    if (ch == ' ')
+    if (ch == ' ') *t++ = '_'; else
       {
-      *t++ = '_';
-      first_byte = FALSE;
-      }
-    else
-      {
-      t += sprintf(CS t, "=%02X", ch);
+      sprintf(CS t, "=%02X", ch);
+      while (*t != 0) t++;
       coded = TRUE;
-      first_byte = !first_byte;
       }
     }
-  else { *t++ = ch; first_byte = FALSE; }
+  else *t++ = ch;
   }
-
+   
 *t++ = '?';
-*t++ = '=';
+*t++ = '=';  
 *t = 0;
-
-return coded ? buffer : string;
+ 
+return coded? buffer : string;
 }
 
 
@@ -984,13 +975,12 @@ Arguments:
 Returns:       the fixed RFC822 phrase
 */
 
-const uschar *
-parse_fix_phrase(const uschar *phrase, int len, uschar *buffer, int buffer_size)
+uschar *
+parse_fix_phrase(uschar *phrase, int len, uschar *buffer, int buffer_size)
 {
 int ch, i;
 BOOL quoted = FALSE;
-const uschar *s, *end;
-uschar *t, *yield;
+uschar *s, *t, *end, *yield;
 
 while (len > 0 && isspace(*phrase)) { phrase++; len--; }
 if (len > buffer_size/4) return US"Name too long";
@@ -1002,7 +992,7 @@ for (i = 0, s = phrase; i < len; i++, s++)
   if ((*s < 32 && *s != '\t') || *s > 126) break;
 
 if (i < len) return parse_quote_2047(phrase, len, headers_charset, buffer,
-  buffer_size, FALSE);
+  buffer_size);
 
 /* No non-printers; use the RFC 822 quoting rules */
 
@@ -1119,7 +1109,7 @@ while (s < end)
 
         else if (ch == '(')
           {
-          const uschar *ss = s;     /* uschar after '(' */
+          uschar *ss = s;     /* uschar after '(' */
           int level = 1;
           while(ss < end)
             {
@@ -1149,12 +1139,9 @@ while (s < end)
             {
             if (ss >= end) ss--;
             *t++ = '(';
-            if (ss > s)
-              {
-              Ustrncpy(t, s, ss-s);
-              t += ss-s;
-              s = ss;
-              }
+            Ustrncpy(t, s, ss-s);
+            t += ss-s;
+            s = ss;
             }
           }
 
@@ -1248,7 +1235,7 @@ Returns:      FF_DELIVERED      addresses extracted
 
 int
 parse_forward_list(uschar *s, int options, address_item **anchor,
-  uschar **error, const uschar *incoming_domain, uschar *directory,
+  uschar **error, uschar *incoming_domain, uschar *directory,
   error_block **syntax_errors)
 {
 int count = 0;
@@ -1431,7 +1418,7 @@ for (;;)
 
     /* Check file name if required */
 
-    if (directory)
+    if (directory != NULL)
       {
       int len = Ustrlen(directory);
       uschar *p = filename + len;
@@ -1443,53 +1430,16 @@ for (;;)
         return FF_ERROR;
         }
 
-#ifdef EXIM_HAVE_OPENAT
-      /* It is necessary to check that every component inside the directory
-      is NOT a symbolic link, in order to keep the file inside the directory.
-      This is mighty tedious. We open the directory and openat every component,
-      with a flag that fails symlinks. */
-
-      {
-      int fd = open(CS directory, O_RDONLY);
-      if (fd < 0)
-	{
-	*error = string_sprintf("failed to open directory %s", directory);
-	return FF_ERROR;
-	}
-      while (*p)
-	{
-	uschar temp;
-	int fd2;
-	uschar * q = p;
-
-	while (*++p && *p != '/') ;
-	temp = *p;
-	*p = '\0';
-
-	fd2 = openat(fd, CS q, O_RDONLY|O_NOFOLLOW);
-	close(fd);
-	*p = temp;
-	if (fd2 < 0)
-	  {
-          *error = string_sprintf("failed to open %s (component of included "
-            "file); could be symbolic link", filename);
-	  return FF_ERROR;
-	  }
-	fd = fd2;
-	}
-      f = fdopen(fd, "rb");
-      }
-#else
       /* It is necessary to check that every component inside the directory
       is NOT a symbolic link, in order to keep the file inside the directory.
       This is mighty tedious. It is also not totally foolproof in that it
       leaves the possibility of a race attack, but I don't know how to do
       any better. */
 
-      while (*p)
+      while (*p != 0)
         {
         int temp;
-        while (*++p && *p != '/');
+        while (*(++p) != 0 && *p != '/');
         temp = *p;
         *p = 0;
         if (Ulstat(filename, &statbuf) != 0)
@@ -1509,16 +1459,11 @@ for (;;)
           return FF_ERROR;
           }
         }
-#endif
       }
 
-#ifdef EXIM_HAVE_OPENAT
-    else
-#endif
-      /* Open and stat the file */
-      f = Ufopen(filename, "rb");
+    /* Open and stat the file */
 
-    if (!f)
+    if ((f = Ufopen(filename, "rb")) == NULL)
       {
       *error = string_open_failed(errno, "included file %s", filename);
       return FF_INCLUDEFAIL;
@@ -1528,13 +1473,13 @@ for (;;)
       {
       *error = string_sprintf("failed to stat included file %s: %s",
         filename, strerror(errno));
-      (void)fclose(f);
+      fclose(f);
       return FF_INCLUDEFAIL;
       }
 
     /* If directory was checked, double check that we opened a regular file */
 
-    if (directory && (statbuf.st_mode & S_IFMT) != S_IFREG)
+    if (directory != NULL && (statbuf.st_mode & S_IFMT) != S_IFREG)
       {
       *error = string_sprintf("included file %s is not a regular file in "
         "the %s directory", filename, directory);
@@ -1555,20 +1500,21 @@ for (;;)
       {
       *error = string_sprintf("error while reading included file %s: %s",
         filename, strerror(errno));
-      (void)fclose(f);
+      fclose(f);
       return FF_ERROR;
       }
     filebuf[statbuf.st_size] = 0;
-    (void)fclose(f);
+    fclose(f);
 
     addr = NULL;
     frc = parse_forward_list(filebuf, options, &addr,
       error, incoming_domain, directory, syntax_errors);
     if (frc != FF_DELIVERED && frc != FF_NOTDELIVERED) return frc;
 
-    if (addr)
+    if (addr != NULL)
       {
-      for (last = addr; last->next; last = last->next) count++;
+      last = addr;
+      while (last->next != NULL) { count++; last = last->next; }
       last->next = *anchor;
       *anchor = addr;
       count++;
@@ -1713,377 +1659,6 @@ for (;;)
 }
 
 
-/*************************************************
-*            Extract a Message-ID                *
-*************************************************/
-
-/* This function is used to extract message ids from In-Reply-To: and
-References: header lines.
-
-Arguments:
-  str          pointer to the start of the message-id
-  yield        put pointer to the message id (in dynamic memory) here
-  error        put error message here on failure
-
-Returns:       points after the processed message-id or NULL on error
-*/
-
-uschar *
-parse_message_id(uschar *str, uschar **yield, uschar **error)
-{
-uschar *domain = NULL;
-uschar *id;
-
-str = skip_comment(str);
-if (*str != '<')
-  {
-  *error = US"Missing '<' before message-id";
-  return NULL;
-  }
-
-/* Getting a block the size of the input string will definitely be sufficient
-for the answer, but it may also be very long if we are processing a header
-line. Therefore, take care to release unwanted store afterwards. */
-
-id = *yield = store_get(Ustrlen(str) + 1);
-*id++ = *str++;
-
-str = read_addr_spec(str, id, '>', error, &domain);
-
-if (*error == NULL)
-  {
-  if (*str != '>') *error = US"Missing '>' after message-id";
-    else if (domain == NULL) *error = US"domain missing in message-id";
-  }
-
-if (*error != NULL)
-  {
-  store_reset(*yield);
-  return NULL;
-  }
-
-while (*id != 0) id++;
-*id++ = *str++;
-*id++ = 0;
-store_reset(id);
-
-str = skip_comment(str);
-return str;
-}
-
-
-/*************************************************
-*        Parse a fixed digit number              *
-*************************************************/
-
-/* Parse a string containing an ASCII encoded fixed digits number
-
-Arguments:
-  str          pointer to the start of the ASCII encoded number
-  n            pointer to the resulting value
-  digits       number of required digits
-
-Returns:       points after the processed date or NULL on error
-*/
-
-static uschar *
-parse_number(uschar *str, int *n, int digits)
-{
-  *n=0;
-  while (digits--)
-  {
-    if (*str<'0' || *str>'9') return NULL;
-    *n=10*(*n)+(*str++-'0');
-  }
-  return str;
-}
-
-
-/*************************************************
-*        Parse a RFC 2822 day of week            *
-*************************************************/
-
-/* Parse the day of the week from a RFC 2822 date, but do not
-   decode it, because it is only for humans.
-
-Arguments:
-  str          pointer to the start of the day of the week
-
-Returns:       points after the parsed day or NULL on error
-*/
-
-static uschar *
-parse_day_of_week(uschar *str)
-{
-/*
-day-of-week     =       ([FWS] day-name) / obs-day-of-week
-
-day-name        =       "Mon" / "Tue" / "Wed" / "Thu" /
-                        "Fri" / "Sat" / "Sun"
-
-obs-day-of-week =       [CFWS] day-name [CFWS]
-*/
-
-static const uschar *day_name[7]={ US"mon", US"tue", US"wed", US"thu", US"fri", US"sat", US"sun" };
-int i;
-uschar day[4];
-
-str=skip_comment(str);
-for (i=0; i<3; ++i)
-  {
-  if ((day[i]=tolower(*str))=='\0') return NULL;
-  ++str;
-  }
-day[3]='\0';
-for (i=0; i<7; ++i) if (Ustrcmp(day,day_name[i])==0) break;
-if (i==7) return NULL;
-str=skip_comment(str);
-return str;
-}
-
-
-/*************************************************
-*            Parse a RFC 2822 date               *
-*************************************************/
-
-/* Parse the date part of a RFC 2822 date-time, extracting the
-   day, month and year.
-
-Arguments:
-  str          pointer to the start of the date
-  d            pointer to the resulting day
-  m            pointer to the resulting month
-  y            pointer to the resulting year
-
-Returns:       points after the processed date or NULL on error
-*/
-
-static uschar *
-parse_date(uschar *str, int *d, int *m, int *y)
-{
-/*
-date            =       day month year
-
-year            =       4*DIGIT / obs-year
-
-obs-year        =       [CFWS] 2*DIGIT [CFWS]
-
-month           =       (FWS month-name FWS) / obs-month
-
-month-name      =       "Jan" / "Feb" / "Mar" / "Apr" /
-                        "May" / "Jun" / "Jul" / "Aug" /
-                        "Sep" / "Oct" / "Nov" / "Dec"
-
-obs-month       =       CFWS month-name CFWS
-
-day             =       ([FWS] 1*2DIGIT) / obs-day
-
-obs-day         =       [CFWS] 1*2DIGIT [CFWS]
-*/
-
-uschar *c,*n;
-static const uschar *month_name[]={ US"jan", US"feb", US"mar", US"apr", US"may", US"jun", US"jul", US"aug", US"sep", US"oct", US"nov", US"dec" };
-int i;
-uschar month[4];
-
-str=skip_comment(str);
-if ((str=parse_number(str,d,1))==NULL) return NULL;
-if (*str>='0' && *str<='9') *d=10*(*d)+(*str++-'0');
-c=skip_comment(str);
-if (c==str) return NULL;
-else str=c;
-for (i=0; i<3; ++i) if ((month[i]=tolower(*(str+i)))=='\0') return NULL;
-month[3]='\0';
-for (i=0; i<12; ++i) if (Ustrcmp(month,month_name[i])==0) break;
-if (i==12) return NULL;
-str+=3;
-*m=i;
-c=skip_comment(str);
-if (c==str) return NULL;
-else str=c;
-if ((n=parse_number(str,y,4)))
-  {
-  str=n;
-  if (*y<1900) return NULL;
-  *y=*y-1900;
-  }
-else if ((n=parse_number(str,y,2)))
-  {
-  str=skip_comment(n);
-  while (*(str-1)==' ' || *(str-1)=='\t') --str; /* match last FWS later */
-  if (*y<50) *y+=100;
-  }
-else return NULL;
-return str;
-}
-
-
-/*************************************************
-*            Parse a RFC 2822 Time               *
-*************************************************/
-
-/* Parse the time part of a RFC 2822 date-time, extracting the
-   hour, minute, second and timezone.
-
-Arguments:
-  str          pointer to the start of the time
-  h            pointer to the resulting hour
-  m            pointer to the resulting minute
-  s            pointer to the resulting second
-  z            pointer to the resulting timezone (offset in seconds)
-
-Returns:       points after the processed time or NULL on error
-*/
-
-static uschar *
-parse_time(uschar *str, int *h, int *m, int *s, int *z)
-{
-/*
-time            =       time-of-day FWS zone
-
-time-of-day     =       hour ":" minute [ ":" second ]
-
-hour            =       2DIGIT / obs-hour
-
-obs-hour        =       [CFWS] 2DIGIT [CFWS]
-
-minute          =       2DIGIT / obs-minute
-
-obs-minute      =       [CFWS] 2DIGIT [CFWS]
-
-second          =       2DIGIT / obs-second
-
-obs-second      =       [CFWS] 2DIGIT [CFWS]
-
-zone            =       (( "+" / "-" ) 4DIGIT) / obs-zone
-
-obs-zone        =       "UT" / "GMT" /          ; Universal Time
-                                                ; North American UT
-                                                ; offsets
-                        "EST" / "EDT" /         ; Eastern:  - 5/ - 4
-                        "CST" / "CDT" /         ; Central:  - 6/ - 5
-                        "MST" / "MDT" /         ; Mountain: - 7/ - 6
-                        "PST" / "PDT" /         ; Pacific:  - 8/ - 7
-
-                        %d65-73 /               ; Military zones - "A"
-                        %d75-90 /               ; through "I" and "K"
-                        %d97-105 /              ; through "Z", both
-                        %d107-122               ; upper and lower case
-*/
-
-uschar *c;
-
-str=skip_comment(str);
-if ((str=parse_number(str,h,2))==NULL) return NULL;
-str=skip_comment(str);
-if (*str!=':') return NULL;
-++str;
-str=skip_comment(str);
-if ((str=parse_number(str,m,2))==NULL) return NULL;
-c=skip_comment(str);
-if (*str==':')
-  {
-  ++str;
-  str=skip_comment(str);
-  if ((str=parse_number(str,s,2))==NULL) return NULL;
-  c=skip_comment(str);
-  }
-if (c==str) return NULL;
-else str=c;
-if (*str=='+' || *str=='-')
-  {
-  int neg;
-
-  neg=(*str=='-');
-  ++str;
-  if ((str=parse_number(str,z,4))==NULL) return NULL;
-  *z=(*z/100)*3600+(*z%100)*60;
-  if (neg) *z=-*z;
-  }
-else
-  {
-  char zone[5];
-  struct { const char *name; int off; } zone_name[10]=
-  { {"gmt",0}, {"ut",0}, {"est",-5}, {"edt",-4}, {"cst",-6}, {"cdt",-5}, {"mst",-7}, {"mdt",-6}, {"pst",-8}, {"pdt",-7}};
-  int i,j;
-
-  for (i=0; i<4; ++i)
-    {
-    zone[i]=tolower(*(str+i));
-    if (zone[i]<'a' || zone[i]>'z') break;
-    }
-  zone[i]='\0';
-  for (j=0; j<10 && strcmp(zone,zone_name[j].name); ++j);
-  /* Besides zones named in the grammar, RFC 2822 says other alphabetic */
-  /* time zones should be treated as unknown offsets. */
-  if (j<10)
-    {
-    *z=zone_name[j].off*3600;
-    str+=i;
-    }
-  else if (zone[0]<'a' || zone[1]>'z') return 0;
-  else
-    {
-    while ((*str>='a' && *str<='z') || (*str>='A' && *str<='Z')) ++str;
-    *z=0;
-    }
-  }
-return str;
-}
-
-
-/*************************************************
-*          Parse a RFC 2822 date-time            *
-*************************************************/
-
-/* Parse a RFC 2822 date-time and return it in seconds since the epoch.
-
-Arguments:
-  str          pointer to the start of the date-time
-  t            pointer to the parsed time
-
-Returns:       points after the processed date-time or NULL on error
-*/
-
-uschar *
-parse_date_time(uschar *str, time_t *t)
-{
-/*
-date-time       =       [ day-of-week "," ] date FWS time [CFWS]
-*/
-
-struct tm tm;
-int zone;
-extern char **environ;
-char **old_environ;
-static char gmt0[]="TZ=GMT0";
-static char *gmt_env[]={ gmt0, (char*)0 };
-uschar *try;
-
-if ((try=parse_day_of_week(str)))
-  {
-  str=try;
-  if (*str!=',') return 0;
-  ++str;
-  }
-if ((str=parse_date(str,&tm.tm_mday,&tm.tm_mon,&tm.tm_year))==NULL) return NULL;
-if (*str!=' ' && *str!='\t') return NULL;
-while (*str==' ' || *str=='\t') ++str;
-if ((str=parse_time(str,&tm.tm_hour,&tm.tm_min,&tm.tm_sec,&zone))==NULL) return NULL;
-tm.tm_isdst=0;
-old_environ=environ;
-environ=gmt_env;
-*t=mktime(&tm);
-environ=old_environ;
-if (*t==-1) return NULL;
-*t-=zone;
-str=skip_comment(str);
-return str;
-}
-
-
-
-
 /*************************************************
 **************************************************
 *             Stand-alone test program           *
@@ -2152,7 +1727,7 @@ allow_utf8_domains = FALSE;
 
 printf("Testing parse_extract_address with group syntax\n");
 
-f.parse_allow_group = TRUE;
+parse_allow_group = TRUE;
 while (Ufgets(buffer, sizeof(buffer), stdin) != NULL)
   {
   uschar *out;
@@ -2216,26 +1791,6 @@ while (Ufgets(buffer, sizeof(buffer), stdin) != NULL)
   else printf("Failed: %d %s\n", extracted, errmess);
   }
 
-printf("Testing parse_message_id\n");
-
-while (Ufgets(buffer, sizeof(buffer), stdin) != NULL)
-  {
-  uschar *s, *t, *errmess;
-  buffer[Ustrlen(buffer) - 1] = 0;
-  if (buffer[0] == 0) break;
-  s = buffer;
-  while (*s != 0)
-    {
-    s = parse_message_id(s, &t, &errmess);
-    if (errmess != NULL)
-      {
-      printf("Failed: %s\n", errmess);
-      break;
-      }
-    printf("%s\n", t);
-    }
-  }
-
 return 0;
 }
 
