@@ -606,14 +606,16 @@ for (i  = (queue_run_in_order? -1 : 0);
     search_tidyup();
 
     /* Now deliver the message; get the id by cutting the -H off the file
-    name. The return of the process is the yield of deliver_message(). */
+    name. The return of the process is zero if a delivery was attempted. */
 
     set_process_info("running queue: %s", f->text);
     f->text[SPOOL_NAME_LENGTH-2] = 0;
     if ((pid = fork()) == 0)
       {
+      int rc;
       (void)close(pfd[pipe_read]);
-      _exit(deliver_message(f->text, force_delivery, FALSE));
+      rc = deliver_message(f->text, force_delivery, FALSE);
+      _exit(rc == DELIVER_NOT_ATTEMPTED);
       }
     if (pid < 0)
       log_write(0, LOG_MAIN|LOG_PANIC_DIE, "fork of delivery process from "
@@ -626,8 +628,8 @@ for (i  = (queue_run_in_order? -1 : 0);
     set_process_info("running queue: waiting for %s (%d)", f->text, pid);
     while (wait(&status) != pid);
 
-    /* A successful return means a delivery was attempted; turn off the
-    force flag for any subsequent calls unless queue_force is set. */
+    /* A zero return means a delivery was attempted; turn off the force flag
+    for any subsequent calls unless queue_force is set. */
 
     if ((status & 0xffff) == 0) force_delivery = queue_run_force;
 
@@ -1186,7 +1188,11 @@ switch(action)
 
   if (deliver_datafile >= 0) printf("has been removed\n");
     else printf("has been removed or did not exist\n");
-  if (removed) log_write(0, LOG_MAIN, "removed by %s", username);
+  if (removed) 
+    {
+    log_write(0, LOG_MAIN, "removed by %s", username);
+    log_write(0, LOG_MAIN, "Completed"); 
+    } 
   break;
 
 
