@@ -1,10 +1,8 @@
-/* $Cambridge: exim/exim-src/src/routers/ipliteral.c,v 1.10 2009/11/16 19:50:38 nm4 Exp $ */
-
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2004 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -90,7 +88,7 @@ ipliteral_router_entry(
   router_instance *rblock,        /* data for this instantiation */
   address_item *addr,             /* address we are working on */
   struct passwd *pw,              /* passwd entry after check_local_user */
-  int verify,                     /* v_none/v_recipient/v_sender/v_expn */
+  BOOL verify,                    /* TRUE when verifying */
   address_item **addr_local,      /* add it to this if it's local */
   address_item **addr_remote,     /* add it to this if it's remote */
   address_item **addr_new,        /* put new addresses on here */
@@ -102,9 +100,8 @@ ipliteral_router_options_block *ob =
 */
 host_item *h;
 uschar *domain = addr->domain;
-uschar *ip;
 int len = Ustrlen(domain);
-int rc, ipv;
+int rc;
 
 addr_new = addr_new;         /* Keep picky compilers happy */
 addr_succeed = addr_succeed;
@@ -112,20 +109,14 @@ addr_succeed = addr_succeed;
 DEBUG(D_route) debug_printf("%s router called for %s: domain = %s\n",
   rblock->name, addr->address, addr->domain);
 
-/* Check that the domain is an IP address enclosed in square brackets. Remember
-to allow for the "official" form of IPv6 addresses. If not, the router
-declines. Otherwise route to the single IP address, setting the host name to
-"(unnamed)". */
+/* Check that the domain is an IP address enclosed in square brackets. If
+not, the router declines. Otherwise route to the single IP address, setting the
+host name to "(unnamed)". */
 
 if (domain[0] != '[' || domain[len-1] != ']') return DECLINE;
 domain[len-1] = 0;  /* temporarily */
 
-ip = domain + 1;
-if (strncmpic(ip, US"IPV6:", 5) == 0 || strncmpic(ip, US"IPV4:", 5) == 0)
-  ip += 5;
-
-ipv = string_is_ip_address(ip, NULL);
-if (ipv == 0 || (disable_ipv6 && ipv == 6))
+if (!string_is_ip_address(domain+1, NULL))
   {
   domain[len-1] = ']';
   return DECLINE;
@@ -135,10 +126,10 @@ if (ipv == 0 || (disable_ipv6 && ipv == 6))
 but if it is set, it should probably work. */
 
 if (verify_check_this_host(&(rblock->ignore_target_hosts), NULL, domain,
-      ip, NULL) == OK)
+      domain + 1, NULL) == OK)
   {
   DEBUG(D_route)
-      debug_printf("%s is in ignore_target_hosts\n", ip);
+      debug_printf("%s is in ignore_target_hosts\n", domain+1);
   addr->message = US"IP literal host explicitly ignored";
   domain[len-1] = ']';
   return DECLINE;
@@ -149,7 +140,7 @@ if (verify_check_this_host(&(rblock->ignore_target_hosts), NULL, domain,
 h = store_get(sizeof(host_item));
 
 h->next = NULL;
-h->address = string_copy(ip);
+h->address = string_copy(domain+1);
 h->port = PORT_NONE;
 domain[len-1] = ']';   /* restore */
 h->name = string_copy(domain);
