@@ -1,10 +1,8 @@
-/* $Cambridge: exim/exim-src/src/routers/queryprogram.h,v 1.5 2007/01/08 10:50:20 ph10 Exp $ */
-
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2007 */
+/* Copyright (c) University of Cambridge 1995 - 2004 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -34,7 +32,7 @@ extern queryprogram_router_options_block queryprogram_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int queryprogram_router_entry(router_instance *, address_item *,
-  struct passwd *, int, address_item **, address_item **,
+  struct passwd *, BOOL, address_item **, address_item **,
   address_item **, address_item **);
 
 extern void queryprogram_router_init(router_instance *);
