@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) University of Cambridge 1995 - 2004 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -26,8 +26,6 @@ optionlist lmtp_transport_options[] = {
       (void *)offsetof(transport_instance, batch_max) },
   { "command",           opt_stringptr,
       (void *)offsetof(lmtp_transport_options_block, cmd) },
-  { "ignore_quota",      opt_bool,
-      (void *)offsetof(lmtp_transport_options_block, ignore_quota) },
   { "socket",            opt_stringptr,
       (void *)offsetof(lmtp_transport_options_block, skt) },
   { "timeout",           opt_time,
@@ -40,25 +38,13 @@ address can appear in the tables drtables.c. */
 int lmtp_transport_options_count =
   sizeof(lmtp_transport_options)/sizeof(optionlist);
 
-
-#ifdef MACRO_PREDEF
-
-/* Dummy values */
-lmtp_transport_options_block lmtp_transport_option_defaults = {0};
-void lmtp_transport_init(transport_instance *tblock) {}
-BOOL lmtp_transport_entry(transport_instance *tblock, address_item *addr) {return FALSE;}
-
-#else   /*!MACRO_PREDEF*/
-
-
 /* Default private options block for the lmtp transport. */
 
 lmtp_transport_options_block lmtp_transport_option_defaults = {
   NULL,           /* cmd */
   NULL,           /* skt */
   5*60,           /* timeout */
-  0,              /* options */
-  FALSE           /* ignore_quota */
+  0               /* options */
 };
 
 
@@ -117,13 +103,12 @@ Arguments:
   more_errno   from the top address for use with ERRNO_FILTER_FAIL
   buffer       the LMTP response buffer
   yield        where to put a one-digit LMTP response code
-  message      where to put an error message
+  message      where to put an errror message
 
 Returns:       TRUE if a "QUIT" command should be sent, else FALSE
 */
 
-static BOOL
-check_response(int *errno_value, int more_errno, uschar *buffer,
+static BOOL check_response(int *errno_value, int more_errno, uschar *buffer,
   int *yield, uschar **message)
 {
 *yield = '4';    /* Default setting is to give a temporary error */
@@ -154,8 +139,8 @@ end the DATA. */
 
 if (*errno_value == ERRNO_FILTER_FAIL)
   {
-  *message = string_sprintf("transport filter process failed (%d)%s",
-    more_errno,
+  *message = string_sprintf("transport filter process failed (%d)%s", 
+    more_errno, 
     (more_errno == EX_EXECFAILED)? ": unable to execute command" : "");
   return FALSE;
   }
@@ -183,7 +168,7 @@ if (*errno_value == ERRNO_WRITEINCOMPLETE)
 
 if (buffer[0] != 0)
   {
-  const uschar *s = string_printing(buffer);
+  uschar *s = string_printing(buffer);
   *message = string_sprintf("LMTP error after %s: %s", big_buffer, s);
   *yield = buffer[0];
   return TRUE;
@@ -222,23 +207,21 @@ Returns:     TRUE if successful, FALSE if not, with errno set
 */
 
 static BOOL
-lmtp_write_command(int fd, const char *format, ...)
+lmtp_write_command(int fd, char *format, ...)
 {
-gstring gs = { .size = big_buffer_size, .ptr = 0, .s = big_buffer };
-int rc;
+int count, rc;
 va_list ap;
-
 va_start(ap, format);
-if (!string_vformat(&gs, FALSE, CS format, ap))
+if (!string_vformat(big_buffer, big_buffer_size, CS format, ap))
   {
-  va_end(ap);
   errno = ERRNO_SMTPFORMAT;
   return FALSE;
   }
 va_end(ap);
-DEBUG(D_transport|D_v) debug_printf("  LMTP>> %s", string_from_gstring(&gs));
-rc = write(fd, gs.s, gs.ptr);
-gs.ptr -= 2; string_from_gstring(&gs); /* remove \r\n for debug and error message */
+count = Ustrlen(big_buffer);
+DEBUG(D_transport|D_v) debug_printf("  LMTP>> %s", big_buffer);
+rc = write(fd, big_buffer, count);
+big_buffer[count-2] = 0;     /* remove \r\n for debug and error message */
 if (rc > 0) return TRUE;
 DEBUG(D_transport) debug_printf("write failed: %s\n", strerror(errno));
 return FALSE;
@@ -301,10 +284,10 @@ for (;;)
 
     *readptr = 0;           /* In case nothing gets read */
     sigalrm_seen = FALSE;
-    ALARM(timeout);
+    alarm(timeout);
     rc = Ufgets(readptr, size-1, f);
     save_errno = errno;
-    ALARM_CLR(0);
+    alarm(0);
     errno = save_errno;
 
     if (rc != NULL) break;  /* A line has been read */
@@ -472,9 +455,8 @@ int code, save_errno;
 BOOL send_data;
 BOOL yield = FALSE;
 address_item *addr;
-uschar *igquotstr = US"";
 uschar *sockname = NULL;
-const uschar **argv;
+uschar **argv;
 uschar buffer[256];
 
 DEBUG(D_transport) debug_printf("%s transport entered\n", tblock->name);
@@ -483,29 +465,13 @@ DEBUG(D_transport) debug_printf("%s transport entered\n", tblock->name);
 not both. When a command is specified, call the common function for creating an
 argument list and expanding the items. */
 
-if (ob->cmd)
+if (ob->cmd != NULL)
   {
   DEBUG(D_transport) debug_printf("using command %s\n", ob->cmd);
   sprintf(CS buffer, "%.50s transport", tblock->name);
   if (!transport_set_up_command(&argv, ob->cmd, TRUE, PANIC, addrlist, buffer,
        NULL))
     return FALSE;
-
-  /* If the -N option is set, can't do any more. Presume all has gone well. */
-  if (f.dont_deliver)
-    goto MINUS_N;
-
-/* As this is a local transport, we are already running with the required
-uid/gid and current directory. Request that the new process be a process group
-leader, so we can kill it and all its children on an error. */
-
-  if ((pid = child_open(USS argv, NULL, 0, &fd_in, &fd_out, TRUE)) < 0)
-    {
-    addrlist->message = string_sprintf(
-      "Failed to create child process for %s transport: %s", tblock->name,
-        strerror(errno));
-    return FALSE;
-    }
   }
 
 /* When a socket is specified, expand the string and create a socket. */
@@ -528,11 +494,38 @@ else
         ob->skt, tblock->name, strerror(errno));
     return FALSE;
     }
+  }
 
-  /* If the -N option is set, can't do any more. Presume all has gone well. */
-  if (f.dont_deliver)
-    goto MINUS_N;
+/* If the -N option is set, can't do any more. Presume all has gone well. */
 
+if (dont_deliver)
+  {
+  DEBUG(D_transport)
+    debug_printf("*** delivery by %s transport bypassed by -N option",
+      tblock->name);
+  addrlist->transport_return = OK;
+  return FALSE;
+  }
+
+/* As this is a local transport, we are already running with the required
+uid/gid and current directory. Request that the new process be a process group
+leader, so we can kill it and all its children on an error. */
+
+if (ob->cmd != NULL)
+  {
+  if ((pid = child_open(argv, NULL, 0, &fd_in, &fd_out, TRUE)) < 0)
+    {
+    addrlist->message = string_sprintf(
+      "Failed to create child process for %s transport: %s", tblock->name,
+        strerror(errno));
+    return FALSE;
+    }
+  }
+
+/* For a socket, try to make the connection */
+
+else
+  {
   sockun.sun_family = AF_UNIX;
   sprintf(sockun.sun_path, "%.*s", (int)(sizeof(sockun.sun_path)-1), sockname);
   if(connect(fd_out, (struct sockaddr *)(&sockun), sizeof(sockun)) == -1)
@@ -544,7 +537,6 @@ else
     }
   }
 
-
 /* Make the output we are going to read into a file. */
 
 out = fdopen(fd_out, "rb");
@@ -567,27 +559,13 @@ if (!lmtp_write_command(fd_in, "%s %s\r\n", "LHLO",
 if (!lmtp_read_response(out, buffer, sizeof(buffer), '2',
      timeout)) goto RESPONSE_FAILED;
 
-/* If the ignore_quota option is set, note whether the server supports the
-IGNOREQUOTA option, and if so, set an appropriate addition for RCPT. */
-
-if (ob->ignore_quota)
-  igquotstr = (pcre_exec(regex_IGNOREQUOTA, NULL, CS buffer,
-    Ustrlen(CS buffer), 0, PCRE_EOPT, NULL, 0) >= 0)? US" IGNOREQUOTA" : US"";
-
 /* Now the envelope sender */
 
 if (!lmtp_write_command(fd_in, "MAIL FROM:<%s>\r\n", return_path))
   goto WRITE_FAILED;
 
 if (!lmtp_read_response(out, buffer, sizeof(buffer), '2', timeout))
-  {
-  if (errno == 0 && buffer[0] == '4')
-    {
-    errno = ERRNO_MAIL4XX;
-    addrlist->more_errno |= ((buffer[1] - '0')*10 + buffer[2] - '0') << 8;
-    }
   goto RESPONSE_FAILED;
-  }
 
 /* Next, we hand over all the recipients. Some may be permanently or
 temporarily rejected; others may be accepted, for now. */
@@ -595,8 +573,8 @@ temporarily rejected; others may be accepted, for now. */
 send_data = FALSE;
 for (addr = addrlist; addr != NULL; addr = addr->next)
   {
-  if (!lmtp_write_command(fd_in, "RCPT TO:<%s>%s\r\n",
-       transport_rcpt_address(addr, tblock->rcpt_include_affixes), igquotstr))
+  if (!lmtp_write_command(fd_in, "RCPT TO:<%s>\r\n",
+       transport_rcpt_address(addr, tblock->rcpt_include_affixes)))
     goto WRITE_FAILED;
   if (lmtp_read_response(out, buffer, sizeof(buffer), '2', timeout))
     {
@@ -608,11 +586,11 @@ for (addr = addrlist; addr != NULL; addr = addr->next)
     if (errno != 0 || buffer[0] == 0) goto RESPONSE_FAILED;
     addr->message = string_sprintf("LMTP error after %s: %s", big_buffer,
       string_printing(buffer));
-    setflag(addr, af_pass_message);   /* Allow message to go to user */
     if (buffer[0] == '5') addr->transport_return = FAIL; else
       {
+      int bincode = (buffer[1] - '0')*10 + buffer[2] - '0';
       addr->basic_errno = ERRNO_RCPT4XX;
-      addr->more_errno |= ((buffer[1] - '0')*10 + buffer[2] - '0') << 8;
+      addr->more_errno |= bincode << 8;
       }
     }
   }
@@ -622,24 +600,10 @@ for (addr = addrlist; addr != NULL; addr = addr->next)
 if (send_data)
   {
   BOOL ok;
-  transport_ctx tctx = {
-    {fd_in},
-    tblock,
-    addrlist,
-    US".", US"..",
-    ob->options
-  };
 
   if (!lmtp_write_command(fd_in, "DATA\r\n")) goto WRITE_FAILED;
   if (!lmtp_read_response(out, buffer, sizeof(buffer), '3', timeout))
-    {
-    if (errno == 0 && buffer[0] == '4')
-      {
-      errno = ERRNO_DATA4XX;
-      addrlist->more_errno |= ((buffer[1] - '0')*10 + buffer[2] - '0') << 8;
-      }
     goto RESPONSE_FAILED;
-    }
 
   sigalrm_seen = FALSE;
   transport_write_timeout = timeout;
@@ -648,7 +612,9 @@ if (send_data)
     debug_printf("  LMTP>> writing message and terminating \".\"\n");
 
   transport_count = 0;
-  ok = transport_write_message(&tctx, 0);
+  ok = transport_write_message(addrlist, fd_in, ob->options, 0,
+        tblock->add_headers, tblock->remove_headers, US".", US"..",
+        tblock->rewrite_rules, tblock->rewrite_existflags);
 
   /* Failure can either be some kind of I/O disaster (including timeout),
   or the failure of a transport filter or the expansion of added headers. */
@@ -671,15 +637,8 @@ if (send_data)
     if (addr->transport_return != PENDING_OK) continue;
 
     if (lmtp_read_response(out, buffer, sizeof(buffer), '2', timeout))
-      {
       addr->transport_return = OK;
-      if (LOGGING(smtp_confirmation))
-        {
-        const uschar *s = string_printing(buffer);
-	/* de-const safe here as string_printing known to have alloc'n'copied */
-        addr->message = (s == buffer)? US string_copy(s) : US s;
-        }
-      }
+
     /* If the response has failed badly, use it for all the remaining pending
     addresses and give up. */
 
@@ -704,15 +663,9 @@ if (send_data)
 
     else
       {
-      if (buffer[0] == '4')
-        {
-        addr->basic_errno = ERRNO_DATA4XX;
-        addr->more_errno |= ((buffer[1] - '0')*10 + buffer[2] - '0') << 8;
-        }
       addr->message = string_sprintf("LMTP error after %s: %s", big_buffer,
         string_printing(buffer));
       addr->transport_return = (buffer[0] == '5')? FAIL : DEFER;
-      setflag(addr, af_pass_message);   /* Allow message to go to user */
       }
     }
   }
@@ -730,15 +683,13 @@ goto RETURN;
 
 /* Come here if any call to read_response, other than a response after the data
 phase, failed. Put the error in the top address - this will be replicated
-because the yield is still FALSE. (But omit ETIMEDOUT, as there will already be
-a suitable message.) Analyse the error, and if if isn't too bad, send a QUIT
-command. Wait for the response with a short timeout, so we don't wind up this
-process before the far end has had time to read the QUIT. */
+because the yield is still FALSE. Analyse the error, and if if isn't too bad,
+send a QUIT command. Wait for the response with a short timeout, so we don't
+wind up this process before the far end has had time to read the QUIT. */
 
 RESPONSE_FAILED:
 
 save_errno = errno;
-if (errno != ETIMEDOUT && errno != 0) addrlist->basic_errno = errno;
 addrlist->message = NULL;
 
 if (check_response(&save_errno, addrlist->more_errno,
@@ -787,22 +738,13 @@ RETURN:
 
 (void)child_close(pid, timeout);
 
-if (fd_in >= 0) (void)close(fd_in);
-if (fd_out >= 0) (void)fclose(out);
+if (fd_in >= 0) (void) close(fd_in);
+if (fd_out >= 0) (void) fclose(out);
 
 DEBUG(D_transport)
   debug_printf("%s transport yields %d\n", tblock->name, yield);
 
 return yield;
-
-
-MINUS_N:
-  DEBUG(D_transport)
-    debug_printf("*** delivery by %s transport bypassed by -N option",
-      tblock->name);
-  addrlist->transport_return = OK;
-  return FALSE;
 }
 
-#endif	/*!MACRO_PREDEF*/
 /* End of transport/lmtp.c */
